package edu.uoc.tfg.yourlovelypet;

import static androidx.test.espresso.Espresso.onView;
import static androidx.test.espresso.action.ViewActions.click;
import static androidx.test.espresso.action.ViewActions.closeSoftKeyboard;
import static androidx.test.espresso.action.ViewActions.typeText;
import static androidx.test.espresso.assertion.ViewAssertions.matches;
import static androidx.test.espresso.intent.Intents.intended;
import static androidx.test.espresso.intent.matcher.IntentMatchers.hasComponent;
import static androidx.test.espresso.intent.matcher.IntentMatchers.hasExtra;
import static androidx.test.espresso.intent.matcher.IntentMatchers.hasExtraWithKey;
import static androidx.test.espresso.matcher.ViewMatchers.isDisplayed;
import static androidx.test.espresso.matcher.ViewMatchers.withId;
import static androidx.test.espresso.matcher.ViewMatchers.withText;
import static org.hamcrest.Matchers.allOf;
import androidx.test.espresso.action.ViewActions;
import androidx.test.espresso.intent.Intents;
import androidx.test.ext.junit.rules.ActivityScenarioRule;
import androidx.test.ext.junit.runners.AndroidJUnit4;
import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;
import edu.uoc.tfg.yourlovelypet.views.LoginActivity;
import edu.uoc.tfg.yourlovelypet.views.MainActivity;

@RunWith(AndroidJUnit4.class)
public class UserActionsTest {

    private static final String TEST_USER_EMAIL = "test@gmail.com";
    private static final String TEST_USER_PASSWORD = "123456";
    private static final String TEST_USER_NAME = "testName";
    private static final String TEST_USER_ID = "M49IIwaiIeXzvlGLP5Jdlop2SNd2";
    private static final long WAIT_LONG = 4000L;

    @Rule
    public ActivityScenarioRule<LoginActivity> loginRule =
            new ActivityScenarioRule<>(LoginActivity.class);

    @Before
    public void setUp() {
        Intents.init();
    }

    @After
    public void tearDown() {
        Intents.release();
    }

    @Test
    public void user_actions() throws InterruptedException {

        // Insert test user credentials
        onView(withId(R.id.edit_email)).perform(typeText(TEST_USER_EMAIL), closeSoftKeyboard());
        onView(withId(R.id.edit_password)).perform(typeText(TEST_USER_PASSWORD), closeSoftKeyboard());

        // Check if test user credential matches with correct values
        onView(withId(R.id.edit_email)).check(matches(withText(TEST_USER_EMAIL)));
        onView(withId(R.id.edit_password)).check(matches(withText(TEST_USER_PASSWORD)));

        // Perform login action
        onView(withId(R.id.buttonLogin)).perform(ViewActions.click());

        // Wait until welcome dialog dismiss and redirect to MainActivity
        Thread.sleep(WAIT_LONG);

        // Check MainActivity is present and has right userId passed
        intended(allOf(
                hasComponent(MainActivity.class.getName()),
                hasExtraWithKey("userId"),
                hasExtra("userId", TEST_USER_ID)));

        // Check user test name is present (showed at homeFragment)
        onView(withId(R.id.textViewUserName)).check(matches(withText(TEST_USER_NAME)));

        // Test bottom navigation bar: ____________________________________________________________

            // Check if bottom navigation bar has right components
            onView(withId(R.id.home)).check(matches(isDisplayed()));
            onView(withId(R.id.search)).check(matches(isDisplayed()));
            onView(withId(R.id.board)).check(matches(isDisplayed()));
            onView(withId(R.id.products)).check(matches(isDisplayed()));
            onView(withId(R.id.user)).check(matches(isDisplayed()));

            // Check navigation to user
            onView(withId(R.id.user)).perform(click());
            onView(withId(R.id.logout)).check(matches(isDisplayed()));

            // Check navigation to search
            onView(withId(R.id.search)).perform(click());
            onView(withId(R.id.textView)).check(matches(withText(R.string.search_title)));

            // Check navigation to board
            onView(withId(R.id.board)).perform(click());
            onView(withId(R.id.textView)).check(matches(withText(R.string.board_title)));

            // Check navigation to products
            onView(withId(R.id.products)).perform(click());
            onView(withId(R.id.textView)).check(matches(withText(R.string.products_title)));

            // Check navigation to home
            onView(withId(R.id.home)).perform(click());
            onView(withId(R.id.textView)).check(matches(withText(R.string.home_title)));

        // Test logout process: ___________________________________________________________________

            // Navigate to user
            onView(withId(R.id.user)).perform(click());

            // Init logout operation
            onView(withId(R.id.logout)).perform(click());

            // Check is logout confirm dialog is present
            onView(withId(R.id.textView)).check(matches(withText(R.string.frag_logout_confirm)));

            // Confirm logout
            onView(withId(R.id.buttonYes)).perform(click());

            // Check is logout bye dialog is present
            onView(withId(R.id.textView1)).check(matches(withText(R.string.frag_logout_bye_text1)));
    }
}