package edu.uoc.tfg.yourlovelypet.repository;

import androidx.annotation.NonNull;
import androidx.lifecycle.MutableLiveData;
import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.auth.EmailAuthProvider;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import java.util.Objects;

public class FirebaseAuthRepository {
    private static FirebaseAuthRepository instance;
    private FirebaseAuth firebaseAuth;

    public FirebaseAuthRepository() {
        this.firebaseAuth = FirebaseAuth.getInstance();
    }
    public static synchronized FirebaseAuthRepository getInstance() {
        if (instance == null) {
            instance = new FirebaseAuthRepository();
        }
        return instance;
    }
    public interface Callback<T> {
        void onSuccess(T result);
        void onFailure(String error);
    }
    public MutableLiveData<FirebaseUser> getCurrentUser() {
        MutableLiveData<FirebaseUser> userLiveData = new MutableLiveData<>();
        FirebaseUser currentUser = firebaseAuth.getCurrentUser();
        userLiveData.setValue(currentUser);
        return userLiveData;
    }
    public void login(String email, String password, Callback<FirebaseUser> callback) {
        firebaseAuth.signInWithEmailAndPassword(email, password)
                .addOnSuccessListener(new OnSuccessListener<AuthResult>() {
                    @Override
                    public void onSuccess(AuthResult authResult) {
                        FirebaseUser user = firebaseAuth.getCurrentUser();
                        callback.onSuccess(user);
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        callback.onFailure(e.getMessage());
                    }
                });
    }
    public void logout(Callback<Void> callback) {
        firebaseAuth.signOut();
        if (firebaseAuth.getCurrentUser() == null) {
            callback.onSuccess(null);
        } else {
            callback.onFailure("Logout failed");
        }
    }
    public void signup(String email, String password, Callback<FirebaseUser> callback) {
        firebaseAuth.createUserWithEmailAndPassword(email, password)
                .addOnSuccessListener(new OnSuccessListener<AuthResult>() {
                    @Override
                    public void onSuccess(AuthResult authResult) {
                        FirebaseUser user = firebaseAuth.getCurrentUser();
                        callback.onSuccess(user);
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        callback.onFailure(e.getMessage());
                    }
                });
    }
    public void reAuthenticateUser(FirebaseUser user, String password, Callback<Void> callback) {
        user.reauthenticate(EmailAuthProvider.getCredential(Objects.requireNonNull(user.getEmail()), password))
                .addOnSuccessListener(new OnSuccessListener<Void>() {
                    @Override
                    public void onSuccess(Void unused) {
                        callback.onSuccess(null);
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        callback.onFailure(e.getMessage());
                    }
                });
    }
    public void updateUserPassword(FirebaseUser user, String newPassword, Callback<Void> callback) {
        user.updatePassword(newPassword)
                .addOnSuccessListener(new OnSuccessListener<Void>() {
                    @Override
                    public void onSuccess(Void unused) {
                        callback.onSuccess(null);
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        callback.onFailure(e.getMessage());
                    }
                });
    }
    public void verifyBeforeUpdateEmail(FirebaseUser user, String newEmail, Callback<Void> callback) {
        user.verifyBeforeUpdateEmail(newEmail)
                .addOnSuccessListener(new OnSuccessListener<Void>() {
                    @Override
                    public void onSuccess(Void unused) {
                        callback.onSuccess(null);
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        callback.onFailure(e.getMessage());
                    }
                });
    }
}