package edu.uoc.tfg.yourlovelypet.repository;

import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.storage.FirebaseStorage;
import com.google.firebase.storage.StorageReference;
import android.net.Uri;
import android.util.Log;
import androidx.annotation.NonNull;

public class FirestorageRepository {

    private static final String TAG = "[FireStorageRepository]";
    private static FirestorageRepository instance;
    private final FirebaseStorage firebaseStorage;
    private final StorageReference storageReference;

    public FirestorageRepository() {
        firebaseStorage = FirebaseStorage.getInstance();
        storageReference = firebaseStorage.getReference();
    }
    public static synchronized FirestorageRepository getInstance() {
        if (instance == null) {
            instance = new FirestorageRepository();
        }
        return instance;
    }
    public interface Callback<T> {
        void onSuccess(T result);
        void onFailure(String error);
    }
    public void deleteImage(String userId, String petId, String filename, Callback<Void> callback) {
        storageReference.child(userId + "/" + petId + "/" + filename).delete()
                .addOnSuccessListener(new OnSuccessListener<Void>() {
                    @Override
                    public void onSuccess(Void unused) {
                        Log.d(TAG,"Deleted from firestorage, image: " + filename + " from pet: " + petId + " from user: " + userId);
                        callback.onSuccess(unused);
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        Log.e(TAG,"Error deleting from firestorage, image: " + filename + " from pet: " + petId + " from user: " + userId + " : ", e);
                        callback.onFailure(e.getMessage());
                    }
                });
    }
    public void getUrl(String path, Callback<Uri> callback) {
        storageReference.child(path).getDownloadUrl()
                .addOnSuccessListener(new OnSuccessListener<Uri>() {
                    @Override
                    public void onSuccess(Uri uri) {
                        Log.d(TAG,"Got image uri from path: " + path);
                        callback.onSuccess(uri);
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        Log.e(TAG,"Error getting image uri from path: " + path, e);
                        callback.onFailure(e.getMessage());
                    }
                });
    }
    public void uploadImage(Uri imageUri, String path, Callback<Uri> callback) {
        StorageReference imageRef = storageReference.child(path);
        imageRef.putFile(imageUri)
                .addOnSuccessListener(taskSnapshot -> imageRef.getDownloadUrl()
                        .addOnSuccessListener(new OnSuccessListener<Uri>() {
                            @Override
                            public void onSuccess(Uri uri) {
                                Log.d(TAG,"Image uploaded at: " + path);
                                callback.onSuccess(uri);
                            }
                        })
                        .addOnFailureListener(new OnFailureListener() {
                            @Override
                            public void onFailure(@NonNull Exception e) {
                                Log.e(TAG,"Error getting image uri: ", e);
                                callback.onFailure(e.getMessage());
                            }
                        }))
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        Log.e(TAG,"Error uploading image to path: " + path, e);
                        callback.onFailure(e.getMessage());
                    }
                });
    }
}
