package edu.uoc.tfg.yourlovelypet.viewModel;

import android.app.Application;
import android.content.Context;
import android.content.SharedPreferences;
import android.util.Log;
import androidx.lifecycle.AndroidViewModel;
import androidx.lifecycle.LiveData;
import androidx.lifecycle.MutableLiveData;
import com.google.firebase.auth.FirebaseUser;
import edu.uoc.tfg.yourlovelypet.model.User;
import edu.uoc.tfg.yourlovelypet.repository.FirebaseAuthRepository;
import edu.uoc.tfg.yourlovelypet.repository.FirestoreRepository;
import edu.uoc.tfg.yourlovelypet.utils.Result;

public class LoginViewModel extends AndroidViewModel {
    // extends from AndroidViewModel instead ViewModel
    // to access application context and use sharedPreferences

    private static final String TAG = "[LoginViewModel]";
    private FirestoreRepository firestoreRepository;
    private FirebaseAuthRepository authRepository;

    // Live data to observers at login operation
    private MutableLiveData<Result> loginResult;
    private MutableLiveData<User> user;

    public LoginViewModel(Application application) {
        super(application);
        firestoreRepository = FirestoreRepository.getInstance();
        authRepository = FirebaseAuthRepository.getInstance();
        loginResult = new MutableLiveData<>();
        user = new MutableLiveData<>();
    }

    public LiveData<Result> getLoginResult() {
        return loginResult;
    }
    public LiveData<User> getUser() {
        return user;
    }

    public void login(String email, String password) {
        authRepository.login(email, password, new FirebaseAuthRepository.Callback<FirebaseUser>() {

            @Override
            public void onSuccess(FirebaseUser result) {

                // get current user data (to show name at welcome again message)
                firestoreRepository.getUser(result.getUid(), new FirestoreRepository.Callback<User>() {
                    @Override
                    public void onSuccess(User userResult) {
                        user.setValue(userResult);
                    }

                    @Override
                    public void onFailure(String error) {
                        Log.e(TAG,"Error getting logged user: " + error);
                    }
                });

                saveUserIdToPreferences(result.getUid()); // save userId to sharedPreferences
                loginResult.setValue(new Result(true, result.getUid()));
            }

            @Override
            public void onFailure(String error) {
                loginResult.setValue(new Result(false, error, true));
            }
        });
    }

    private void saveUserIdToPreferences(String userId) {
        SharedPreferences sharedPreferences = getApplication().getSharedPreferences("MyAppPrefs", Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = sharedPreferences.edit();
        editor.putString("userId", userId);
        editor.apply();
    }
}
