package edu.uoc.tfg.yourlovelypet.viewModel;

import android.app.Application;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.res.Configuration;
import android.content.res.Resources;
import android.net.Uri;
import android.util.DisplayMetrics;
import android.util.Log;
import androidx.lifecycle.AndroidViewModel;
import androidx.lifecycle.LiveData;
import androidx.lifecycle.MutableLiveData;
import com.google.firebase.auth.FirebaseUser;
import java.util.List;
import java.util.Locale;
import java.util.Objects;
import edu.uoc.tfg.yourlovelypet.model.Pet;
import edu.uoc.tfg.yourlovelypet.model.Post;
import edu.uoc.tfg.yourlovelypet.model.Product;
import edu.uoc.tfg.yourlovelypet.model.User;
import edu.uoc.tfg.yourlovelypet.repository.FirebaseAuthRepository;
import edu.uoc.tfg.yourlovelypet.repository.FirestorageRepository;
import edu.uoc.tfg.yourlovelypet.repository.FirestoreRepository;

public class MainViewModel extends AndroidViewModel {
    // extends from AndroidViewModel instead ViewModel
    // to access application context and use sharedPreferences

    private static final String TAG = "[MainViewModel]";
    private FirestoreRepository firestoreRepository;
    private FirestorageRepository firestorageRepository;
    private FirebaseAuthRepository authRepository;
    private MutableLiveData<User> user;
    private MutableLiveData<List<Pet>> pets;
    private MutableLiveData<List<Product>> products;
    private MutableLiveData<List<Post>> posts;
    private MutableLiveData<String> userLanguage;
    private String userId;

    // Live data operations ("callbacks" to MainActivity and some special Views) start _________________

    // Live data to observers at addPostUser operation
    private MutableLiveData<Boolean> addPostSuccess = new MutableLiveData<>();
    private MutableLiveData<String> addPostError = new MutableLiveData<>();

    // Live data to observers at addPetUser operation
    private MutableLiveData<Boolean> addPetSuccess = new MutableLiveData<>();
    private MutableLiveData<String> addPetError = new MutableLiveData<>();

    // Live data to observers at updateUserLanguage operation
    private MutableLiveData<Boolean> updateUserLanguageSuccess = new MutableLiveData<>();
    private MutableLiveData<String> updateUserLanguageError = new MutableLiveData<>();

    // Live data to observers at deleteImage operation
    private MutableLiveData<Boolean> deleteImageSucces = new MutableLiveData<>();
    private MutableLiveData<String> deleteImageError = new MutableLiveData<>();

    // Live data to observers at updatePet operation
    private MutableLiveData<Boolean> updatePetSuccess = new MutableLiveData<>();
    private MutableLiveData<String> updatePetError = new MutableLiveData<>();

    // Live data to observers at deletePet operation
    private MutableLiveData<Boolean> deletePetSuccess = new MutableLiveData<>();
    private MutableLiveData<String> deletePetError = new MutableLiveData<>();

    // Live data to observers at updateUserName operation
    private MutableLiveData<Boolean> updateUserNameSuccess = new MutableLiveData<>();
    private MutableLiveData<String> updateUserNameError = new MutableLiveData<>();

    // Live data to observers at updateUserEmail operation
    private MutableLiveData<Boolean> updateUserEmailSuccess = new MutableLiveData<>();
    private MutableLiveData<String> updateUserEmailError = new MutableLiveData<>();

    // Live data to observers at updateUserPassword operation
    private MutableLiveData<Boolean> updateUserPasswordSuccess = new MutableLiveData<>();
    private MutableLiveData<String> updateUserPasswordError = new MutableLiveData<>();

    // Live data to observers at addImage operation
    private MutableLiveData<Boolean> addImageSuccess = new MutableLiveData<>();
    private MutableLiveData<String> addImageError = new MutableLiveData<>();

    // Live data to observers at logout operation
    private MutableLiveData<Boolean> logoutSuccess = new MutableLiveData<>();
    private MutableLiveData<String> logoutError = new MutableLiveData<>();

    // Live data to observers at updateProfileImage operation
    private MutableLiveData<Boolean> updateProfileImageSucces = new MutableLiveData<>();
    private MutableLiveData<String> updateProfileImageError = new MutableLiveData<>();

    // Live data to observers at addProduct operation
    private MutableLiveData<Boolean> addProductSuccess = new MutableLiveData<>();
    private MutableLiveData<String> addProductError = new MutableLiveData<>();

    // Live data to observers at deleteProduct operation
    private MutableLiveData<Boolean> deleteProductSuccess = new MutableLiveData<>();
    private MutableLiveData<String> deleteProductError = new MutableLiveData<>();

    // Getters of live data to observers at addPost operation
    public LiveData<Boolean> getAddPostSuccess() {
        return addPostSuccess;
    }
    public LiveData<String> getAddPostError() {
        return addPostError;
    }
    public void resetAddPostSuccess() {
        addPostSuccess.setValue(false);
    }
    public void resetAddPostError() {
        addPostError.setValue(null);
    }

    // Getters of live data to observers at addPet operation
    public LiveData<Boolean> getAddPetSuccess() {
        return addPetSuccess;
    }
    public LiveData<String> getAddPetError() {
        return addPetError;
    }
    public void resetAddPetSuccess() {
        addPetSuccess.setValue(false);
    }
    public void resetAddPetError() {
        addPetError.setValue(null);
    }

    // Getters of live data to observers at updateUserLanguage operation
    public LiveData<Boolean> getUpdateUserLanguageSuccess() {
        return updateUserLanguageSuccess;
    }
    public LiveData<String> getUpdateUserLanguageError() {
        return updateUserLanguageError;
    }
    public void resetUpdateUserLanguageSuccess() {
        updateUserLanguageSuccess.setValue(false);
    }
    public void resetUpdateUserLanguageError() {
        updateUserLanguageError.setValue(null);
    }

    // Getters of live data to observers at deleteImage operation
    public LiveData<Boolean> getDeleteImageSuccess() {
        return deleteImageSucces;
    }
    public LiveData<String> getDeleteImageError() {
        return deleteImageError;
    }
    public void resetDeleteImageSuccess() {
        deleteImageSucces.setValue(false);
    }
    public void resetDeleteImageError() {
        deleteImageError.setValue(null);
    }

    // Getters of live data to observers at updatePet operation
    public LiveData<Boolean> getUpdatePetSuccess() {
        return updatePetSuccess;
    }
    public LiveData<String> getUpdatePetError() {
        return updatePetError;
    }
    public void resetUpdatePetSuccess() {
        updatePetSuccess.setValue(false);
    }
    public void resetUpdatePetError() {
        updatePetError.setValue(null);
    }

    // Getters of live data to observers at deletePet operation
    public LiveData<Boolean> getDeletePetSuccess() {
        return deletePetSuccess;
    }
    public LiveData<String> getDeletePetError() {
        return deletePetError;
    }
    public void resetDeletePetSuccess() {
        deletePetSuccess.setValue(false);
    }
    public void resetDeletePetError() {
        deletePetError.setValue(null);
    }

    // Getters of live data to observers at updateUserNameSuccess operation
    public LiveData<Boolean> getUpdateUserNameSuccess() {
        return updateUserNameSuccess;
    }
    public LiveData<String> getUpdateUserNameError() {
        return updateUserNameError;
    }
    public void resetUpdateUserNameSuccess() {
        updateUserNameSuccess.setValue(false);
    }
    public void resetUpdateUserNameError() {
        updateUserNameError.setValue(null);
    }

    // Getters of live data to observers at updateUserEmailSuccess operation
    public LiveData<Boolean> getUpdateUserEmailSuccess() {
        return updateUserEmailSuccess;
    }
    public LiveData<String> getUpdateUserEmailError() {
        return updateUserEmailError;
    }
    public void resetUpdateUserEmailSuccess() {
        updateUserEmailSuccess.setValue(false);
    }
    public void resetUpdateUserEmailError() {
        updateUserEmailError.setValue(null);
    }

    // Getters of live data to observers at updateUserPasswordSuccess operation
    public LiveData<Boolean> getUpdateUserPasswordSuccess() {
        return updateUserPasswordSuccess;
    }
    public LiveData<String> getUpdateUserPasswordError() {
        return updateUserPasswordError;
    }
    public void resetUpdateUserPasswordSuccess() {
        updateUserPasswordSuccess.setValue(false);
    }
    public void resetUpdateUserPasswordError() {
        updateUserPasswordError.setValue(null);
    }

    // Getters of live data to observers at addImageSuccess operation
    public LiveData<Boolean> getAddImageSuccess() {
        return addImageSuccess;
    }
    public LiveData<String> getAddImageError() {
        return addImageError;
    }
    public void resetAddImageSuccess() {
        addImageSuccess.setValue(false);
    }
    public void resetAddImageError() {
        addImageError.setValue(null);
    }

    // Getters of live data to observers at logout operation
    public LiveData<Boolean> getLogoutSuccess() {
        return logoutSuccess;
    }
    public LiveData<String> getLogoutError() {
        return logoutError;
    }
    public void resetLogoutSuccess() {
        logoutSuccess.setValue(false);
    }
    public void resetLogoutError() {
        logoutError.setValue(null);
    }

    // Getters of live data to observers at updateProfileImage operation
    public LiveData<Boolean> getUpdateProfileImageSuccess() {
        return updateProfileImageSucces;
    }
    public LiveData<String> getUpdateProfileImageError() {
        return updateProfileImageError;
    }
    public void resetUpdateProfileImageSuccess() {
        updateProfileImageSucces.setValue(false);
    }
    public void resetUpdateProfileImageError() {
        updateProfileImageError.setValue(null);
    }

    // Getters of live data to observers at addProduct operation
    public LiveData<Boolean> getAddProductSuccess() {
        return addProductSuccess;
    }
    public LiveData<String> getAddProductError() {
        return addProductError;
    }
    public void resetAddProductSuccess() {
        addProductSuccess.setValue(false);
    }
    public void resetAddProductError() {
        addProductError.setValue(null);
    }

    // Getters of live data to observers at deleteProduct operation
    public LiveData<Boolean> getDeleteProductSuccess() {
        return deleteProductSuccess;
    }
    public LiveData<String> getDeleteProductError() {
        return deleteProductError;
    }
    public void resetDeleteProductSuccess() {
        deleteProductSuccess.setValue(false);
    }
    public void resetDeleteProductError() {
        deleteProductError.setValue(null);
    }

    // Live data operations ("callbacks" to Views) end _________________________________________________

    public MainViewModel(Application application, String id) {
        super(application);
        userId = id;
        firestoreRepository = FirestoreRepository.getInstance();
        firestorageRepository = FirestorageRepository.getInstance();
        authRepository = FirebaseAuthRepository.getInstance();
        user = new MutableLiveData<>();
        pets = new MutableLiveData<>();
        products = new MutableLiveData<>();
        posts = new MutableLiveData<>();
        userLanguage = new MutableLiveData<>();

        // Get current user data from repository
        Log.d(TAG,"Current user has this id: " + userId);
        userLanguage = (MutableLiveData<String>) firestoreRepository.getCurrentUserLanguage(userId);
        loadUser();
        loadPets();
        loadProducts();
        loadPosts();
    }

    public void setLocale(String lang, Resources resources) {
        DisplayMetrics displayMetrics = resources.getDisplayMetrics();
        Configuration configuration = resources.getConfiguration();
        Locale locale = new Locale(lang);
        configuration.setLocale(locale);
        resources.updateConfiguration(configuration, displayMetrics);
    }

    public LiveData<User> getCurrentUser() {
        return user;
    }
    public LiveData<List<Pet>> getPetsFromCurrentUser() {
        return pets;
    }
    public LiveData<List<Product>> getProductsFromCurrentUser() {
        return products;
    }
    public LiveData<List<Post>> getPosts() {
          return posts;
    }
    public void updateUserName(User currentUser, String name) {
        if (!name.equals(currentUser.getUserName())) {
            User user = new User(name, currentUser.getUserEmail(), currentUser.getUserLanguage());
            user.setId(currentUser.getId());
            firestoreRepository.updateUser(user, new FirestoreRepository.Callback<Void>() {
                @Override
                public void onSuccess(Void result) {
                    updateUserNameSuccess.setValue(true);
                }

                @Override
                public void onFailure(String error) {
                    updateUserNameError.setValue(error);
                }
            });

        }
    }
    public void updateUserEmail(User currentUser, String newEmail, String password) {

        FirebaseUser firebaseUser = authRepository.getCurrentUser().getValue();
        if (firebaseUser != null) {
            if (!newEmail.equals(currentUser.getUserEmail())) {
                authRepository.reAuthenticateUser(firebaseUser, password, new FirebaseAuthRepository.Callback<Void>() {
                    @Override
                    public void onSuccess(Void result) {
                        Log.d(TAG, "User re-authenticated successfully");
                        authRepository.verifyBeforeUpdateEmail(firebaseUser, newEmail, new FirebaseAuthRepository.Callback<Void>() {
                            @Override
                            public void onSuccess(Void result) {
                                User updatedUser = new User(currentUser.getUserName(), newEmail, currentUser.getUserLanguage());
                                updatedUser.setId(currentUser.getId());
                                firestoreRepository.updateUser(updatedUser, new FirestoreRepository.Callback<Void>() {
                                    @Override
                                    public void onSuccess(Void result) {
                                        Log.d(TAG, "User email updated");
                                        updateUserEmailSuccess.setValue(true);
                                    }

                                    @Override
                                    public void onFailure(String error) {
                                        Log.e(TAG, "Error saving updated user email to database: " + error);
                                        updateUserEmailError.setValue("Error saving updated user email to database: " + error);
                                    }
                                });
                            }

                            @Override
                            public void onFailure(String error) {
                                Log.e(TAG, "Error verifying new email: " + error);
                                updateUserEmailError.setValue("Error verifying new email: " + error);
                            }
                        });
                    }

                    @Override
                    public void onFailure(String error) {
                        Log.e(TAG, "Error re-authenticating user: " + error);
                        updateUserEmailError.setValue("Error re-authenticating user: " + error);
                    }
                });
            }
        }
    }
    public void updateUserPassword(String password, String newPassword) {

        FirebaseUser firebaseUser = authRepository.getCurrentUser().getValue();
        if (firebaseUser != null) {
            if (!password.equals(newPassword)) {
                authRepository.reAuthenticateUser(firebaseUser, password, new FirebaseAuthRepository.Callback<Void>() {
                    @Override
                    public void onSuccess(Void result) {
                        Log.d(TAG, "User re-authenticated successfully");
                        authRepository.updateUserPassword(firebaseUser, newPassword, new FirebaseAuthRepository.Callback<Void>() {
                            @Override
                            public void onSuccess(Void result) {
                                Log.d(TAG, "User password updated successfully");
                                updateUserPasswordSuccess.setValue(true);
                            }

                            @Override
                            public void onFailure(String error) {
                                Log.e(TAG, "Error updating user password: " + error);
                                updateUserPasswordError.setValue("Error updating user password: " + error);
                            }
                        });
                    }

                    @Override
                    public void onFailure(String error) {
                        Log.e(TAG, "Error re-authenticating user: " + error);
                        updateUserPasswordError.setValue("Error re-authenticating user: " + error);
                    }
                });
            }
        }
    }
    public void updateUserLanguage(String language, Resources resources) {
        Log.d(TAG, "Request to update user language to: " + language);
        User newUser = new User(Objects.requireNonNull(user.getValue()).getUserName(), user.getValue().getUserEmail(), language);
        newUser.setId(userId);
        firestoreRepository.updateUserLanguage(newUser, new FirestoreRepository.Callback<Void>() {
            @Override
            public void onSuccess(Void result) {
                updateUserLanguageSuccess.setValue(true);
                setLocale(language, resources);
            }

            @Override
            public void onFailure(String error) {
                updateUserLanguageError.setValue(error);
            }
        });
    }
    public void addPet(Pet pet) {
        firestoreRepository.addPet(pet, userId, new FirestoreRepository.Callback<Pet>() {
            @Override
            public void onSuccess(Pet result) {
                addPetSuccess.setValue(true);
            }

            @Override
            public void onFailure(String error) {
                addPetError.setValue(error);
            }
        });
    }
    public void addPost(Post post) {
        firestoreRepository.addPost(post, new FirestoreRepository.Callback<Post>() {
            @Override
            public void onSuccess(Post result) {
                addPostSuccess.setValue(true);
            }

            @Override
            public void onFailure(String error) {
                addPostError.setValue(error);
            }
        });
    }
    public void addProduct(Product product) {
        firestoreRepository.addProduct(product, userId, new FirestoreRepository.Callback<Product>() {
            @Override
            public void onSuccess(Product result) {
                addProductSuccess.setValue(true);
            }

            @Override
            public void onFailure(String error) {
                addProductError.setValue(error);
            }
        });
    }
    public void updatePet(Pet petToUpdate) {
        firestoreRepository.updatePet(petToUpdate, userId, new FirestoreRepository.Callback<Void>() {
            @Override
            public void onSuccess(Void result) {
                updatePetSuccess.setValue(true);
            }

            @Override
            public void onFailure(String error) {
                updatePetError.setValue(error);
            }
        });
    }
    public void deletePet(String petId) {
        firestoreRepository.deletePet(userId, petId, new FirestoreRepository.Callback<Void>() {
            @Override
            public void onSuccess(Void result) {
                deletePetSuccess.setValue(true);
            }

            @Override
            public void onFailure(String error) {
                deletePetError.setValue(error);
            }
        });
    }
    public void deleteProduct(String productId) {
        firestoreRepository.deleteProduct(userId, productId, new FirestoreRepository.Callback<Void>() {
            @Override
            public void onSuccess(Void result) {
                deleteProductSuccess.setValue(true);
            }

            @Override
            public void onFailure(String error) {
                deleteProductError.setValue(error);
            }
        });
    }
    public Pet getPetById(String id) {
        List<Pet> petList = pets.getValue();
        if (petList != null) {
            for (Pet pet : petList) {
                if (pet.getId().equals((id))) {
                    return pet;
                }
            }
        }
        return null;
    }
    public Post getPostById(String id) {
        List<Post> postList = posts.getValue();
        if (postList != null) {
            for (Post post : postList) {
                if (post.getId().equals((id))) {
                    return post;
                }
            }
        }
        return null;
    }
    public LiveData<List<String>> getGalleryFromUserPet(String petId) {
        return firestoreRepository.getGalleryFromUserPet(userId, petId);
    }
    public void uploadImage(Uri imageUri, String petId, String path, String fileName) {
        String fullPath = path + fileName;
        firestorageRepository.uploadImage(imageUri, fullPath, new FirestorageRepository.Callback<Uri>() {
            @Override
            public void onSuccess(Uri uri) {
                String downloadUrl = uri.toString();
                Log.d(TAG, "Upload successful. Uri: " + downloadUrl);
                // add new document to sub-collection gallery from that pet from this user
                addNewImage(userId, petId, fileName);
            }

            @Override
            public void onFailure(String error) {
                Log.e(TAG, "Upload failed: " + error);
            }
        });
    }
    public void addNewImage(String userId, String petId, String filename) {
        firestoreRepository.addNewImage(userId, petId, filename, new FirestoreRepository.Callback<Void>() {
            @Override
            public void onSuccess(Void result) {
                Log.d(TAG,"Image reference: " + filename + " added to gallery of pet: " + petId + "of user: " + userId);
                addImageSuccess.setValue(true);
            }

            @Override
            public void onFailure(String error) {
                Log.d(TAG,"Error adding image reference: " + filename + " to gallery of pet: " + petId + "of user: " + userId);
                Log.d(TAG, "Error: " + error);
                addImageError.setValue(error);
            }
        });
    }
    public void getUrl(String path, FirestorageRepository.Callback<Uri> callback) {
        firestorageRepository.getUrl(path, callback);
    }
    public void deleteImage(String petId, String imageUrl) {

        String filename = extractFilenameFromUrl(imageUrl);
        Log.d(TAG, "Request to delete image with url: " + imageUrl);
        Log.d(TAG, "Request to delete image: " + filename + " from pet: " + petId + " from user: " + userId);

        firestorageRepository.deleteImage(userId, petId, filename, new FirestorageRepository.Callback<Void>() {
            @Override
            public void onSuccess(Void result) {
                firestoreRepository.deleteImage(userId, petId, filename, new FirestoreRepository.Callback<Void>() {
                    @Override
                    public void onSuccess(Void result) {
                        deleteImageSucces.setValue(true);
                        Log.d(TAG,"Image deleted from firestorage and firestore");
                    }

                    @Override
                    public void onFailure(String error) {
                        deleteImageError.setValue(error);
                        Log.d(TAG,"Image deleted from firestorage but error occurs deleting at firestore: " + error);
                    }
                });
            }
            @Override
            public void onFailure(String error) {
                deleteImageError.setValue(error);
                Log.e(TAG,"Error deleting image at firestorage: " + error);
            }
        });
    }
    private String extractFilenameFromUrl(String url) {
        // Auxiliary method to extract filename from url
        if (url == null || url.trim().isEmpty()) return null;

        try {
            String decodedUrl = url.replace("%2F", "/");
            int lastIndex = decodedUrl.lastIndexOf('/');
            return lastIndex != -1 ? decodedUrl.substring(lastIndex + 1, decodedUrl.indexOf('?')) : null;
        } catch (Exception e) {
            Log.e(TAG,"Error extracting filename from imageUrl: ", e);
        }

        return null;
    }
    public void updateProfileImage(String petId, String imageUrl) {
        String filename = extractFilenameFromUrl(imageUrl);
        Log.d(TAG, "Request to set profile image of pet: " + petId + " this image: " + filename);
        firestoreRepository.updateProfileImage(userId, petId, filename, new FirestoreRepository.Callback<Void>() {

            @Override
            public void onSuccess(Void result) {
                updateProfileImageSucces.setValue(true);
                Log.d(TAG,"Image profile updated of pet: " + petId + " to image: " + filename);
            }

            @Override
            public void onFailure(String error) {
                updateProfileImageError.setValue(error);
                Log.e(TAG,"Error updating profile image of pet: " + petId + " : " + error);
            }
        });
    }
    public int getRepliesNumber(String postId) {
        if ( (posts.getValue() == null) || (posts == null) ) return 0;
        int count = 0;
        for (Post post : posts.getValue()) {
            if (postId.equals(post.getReplyToId())) {
                count++;
            }
        }
        return count;
    }
    public void loadPosts() {
        firestoreRepository.getPosts(new FirestoreRepository.Callback<List<Post>>() {

            @Override
            public void onSuccess(List<Post> result) {
                if (result != null) {
                    posts.setValue(result); // Update the MutableLiveData
                } else {
                    Log.e(TAG,"Posts result is null");
                }
            }

            @Override
            public void onFailure(String error) {
                Log.e(TAG,"Error loading posts: " + error);
            }
        });
    }
    public void loadUser() {
        firestoreRepository.getUser(userId, new FirestoreRepository.Callback<User>() {
            @Override
            public void onSuccess(User result) {
                user.setValue(result); // Update the MutableLiveData
            }

            @Override
            public void onFailure(String error) {
                Log.e(TAG,"Error loading user: " + error);
            }
        });
    }
    public void loadPets() {
        firestoreRepository.getPetsFromUser(userId, new FirestoreRepository.Callback<List<Pet>>() {
            @Override
            public void onSuccess(List<Pet> result) {
                pets.setValue(result); // Update the MutableLiveData
            }

            @Override
            public void onFailure(String error) {
                Log.e(TAG,"Error loading pets: " + error);
            }
        });
    }
    public void loadProducts() {
        firestoreRepository.getProductsFromUser(userId, new FirestoreRepository.Callback<List<Product>>() {
            @Override
            public void onSuccess(List<Product> result) {
                products.setValue(result); // Update the MutableLiveData
            }

            @Override
            public void onFailure(String error) {
                Log.e(TAG,"Error loading products: " + error);
            }
        });
    }
    public void refreshPosts() {
        loadPosts();
    }
    public void logout() {
        authRepository.logout(new FirebaseAuthRepository.Callback<Void>() {
            @Override
            public void onSuccess(Void result) {
                firestoreRepository.clearListeners();
                logoutSuccess.setValue(true);
                Log.d(TAG,"User logout successfully");
            }

            @Override
            public void onFailure(String error) {
                logoutError.setValue(error);
                Log.d(TAG,"Error during user logout: " + error);
            }
        });
    }
    public void clearUserIdFromPreferences() {
        SharedPreferences sharedPreferences = getApplication().getSharedPreferences("MyAppPrefs", Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = sharedPreferences.edit();
        editor.remove("userId");
        editor.apply();
    }
}