package edu.uoc.tfg.yourlovelypet.viewModel;

import android.util.Log;
import androidx.lifecycle.LiveData;
import androidx.lifecycle.MutableLiveData;
import androidx.lifecycle.ViewModel;
import com.google.firebase.auth.FirebaseUser;
import edu.uoc.tfg.yourlovelypet.model.User;
import edu.uoc.tfg.yourlovelypet.repository.FirebaseAuthRepository;
import edu.uoc.tfg.yourlovelypet.repository.FirestoreRepository;
import edu.uoc.tfg.yourlovelypet.utils.Result;

public class SignupViewModel extends ViewModel {

    private static final String TAG = "[SignupViewModel]";
    private FirestoreRepository firestoreRepository;
    private FirebaseAuthRepository authRepository;

    // Live data to observer at signup operation
    private MutableLiveData<Result> signupResult;
    private MutableLiveData<User> user;

    public SignupViewModel() {
        firestoreRepository = FirestoreRepository.getInstance();
        authRepository = FirebaseAuthRepository.getInstance();
        signupResult = new MutableLiveData<>();
        user = new MutableLiveData<>();
    }

    // Getter of live data to observer at signup operation
    public LiveData<Result> getSignupResult() {
        return signupResult;
    }
    public LiveData<User> getUser() {
        return user;
    }

    public void signup(String name, String email, String password, String language) {
        authRepository.signup(email, password, new FirebaseAuthRepository.Callback<FirebaseUser>() {

            @Override
            public void onSuccess(FirebaseUser result) {
                if (result != null) {
                    String userId = result.getUid();
                    Log.d(TAG, "User sign up successfully. New user authentication id is: " + userId);
                    User newUser = new User(name, email, language);
                    newUser.setId(userId);
                    addUser(newUser);
                    // signupResult deepens on addUser operation
                }
            }

            @Override
            public void onFailure(String error) {
                signupResult.setValue(new Result(false, error, true));
            }
        });
    }

    public void addUser(User newUser) {
        Log.d(TAG,"Request to add a new user with id: " + newUser.getId());
        firestoreRepository.addUser(newUser, new FirestoreRepository.Callback<User>(){

            @Override
            public void onSuccess(User result) {
                user.setValue(result);
                // addUser operation defines signup operation result
                signupResult.setValue(new Result(true, newUser.getId()));
            }

            @Override
            public void onFailure(String error) {
                // addUser operation defines signup operation result
                signupResult.setValue(new Result(false, error, true));
            }
        });
    }
}
