package edu.uoc.tfg.yourlovelypet.views;

import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.widget.Toast;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModelProvider;
import androidx.navigation.NavController;
import androidx.navigation.fragment.NavHostFragment;
import androidx.navigation.ui.NavigationUI;
import com.google.android.material.bottomnavigation.BottomNavigationView;
import com.google.firebase.FirebaseApp;
import java.util.Objects;
import edu.uoc.tfg.yourlovelypet.R;
import edu.uoc.tfg.yourlovelypet.databinding.ActivityMainBinding;
import edu.uoc.tfg.yourlovelypet.viewModel.MainViewModel;
import edu.uoc.tfg.yourlovelypet.viewModel.MainViewModelFactory;
import edu.uoc.tfg.yourlovelypet.views.dialogs.LogoutByeDialogFragment;

public class MainActivity extends AppCompatActivity {

    private static final String TAG = "[MainActivity]";
    private static final int SPLASH_DIALOG = 3000;
    private String userId;
    private ActivityMainBinding binding;
    private MainViewModel mainViewModel;
    private NavController bottomNavController;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = ActivityMainBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());

        userId = getIntent().getStringExtra("userId");
        Log.d(TAG,"Current user has this id: " + userId);

        // Instance of shared ViewModel. Sending userId (get from LoginActivity/SignupActivity)
        mainViewModel = new ViewModelProvider(this, new MainViewModelFactory(getApplication(), userId)).get(MainViewModel.class);

        FirebaseApp.initializeApp(this);
        initUI();
        setupObservers();
    }

    private void initUI() {

        // NavHost and NavController for BottomNavigationView
        NavHostFragment navHostFragment = (NavHostFragment) getSupportFragmentManager().findFragmentById(R.id.main);
        bottomNavController = Objects.requireNonNull(navHostFragment).getNavController();
        BottomNavigationView bottomNavView = binding.bottomNavigationView;
        NavigationUI.setupWithNavController(bottomNavView, bottomNavController);

        // Bottom navigation bar
        binding.bottomNavigationView.setOnItemSelectedListener(item -> {
            if (item.getItemId() == R.id.home) {
                bottomNavController.navigate(R.id.homeFragment);
            } else if (item.getItemId() == R.id.search) {
                bottomNavController.navigate(R.id.searchFragment);
            } else if (item.getItemId() == R.id.board) {
                bottomNavController.navigate(R.id.boardFragment);
            } else if (item.getItemId() == R.id.products) {
                bottomNavController.navigate(R.id.productsFragment);
            } else if (item.getItemId() == R.id.user) {
                bottomNavController.navigate(R.id.userFragment);
            }
            return true;
        });
    }

    private void setupObservers() {

        // Observe addPet operation success LiveData
        mainViewModel.getAddPetSuccess().observe(this, new Observer<Boolean>() {
            @Override
            public void onChanged(Boolean success) {
                if (success != null && success) {
                    Log.d(TAG, "New pet data saved at database");
                    mainViewModel.resetAddPetSuccess(); // Reset Livedata value
                    Toast.makeText(MainActivity.this, getString(R.string.mvm_pet_added_success), Toast.LENGTH_SHORT).show();
                }
            }
        });

        // Observe addPet operation error LiveData
        mainViewModel.getAddPetError().observe(this, new Observer<String>() {
            @Override
            public void onChanged(String error) {
                if (error != null) {
                    Log.d(TAG, "Error adding pet: " + error);
                    mainViewModel.resetAddPetError(); // Reset Livedata value
                    Toast.makeText(MainActivity.this, getString(R.string.mvm_pet_added_error), Toast.LENGTH_SHORT).show();
                }
            }
        });

        // Observe deletePet operation success LiveData
        mainViewModel.getDeletePetSuccess().observe(this, new Observer<Boolean>() {
            @Override
            public void onChanged(Boolean success) {
                if (success != null && success) {
                    Log.d(TAG, "Pet data deleted");
                    mainViewModel.resetDeletePetSuccess(); // Reset Livedata value
                    Toast.makeText(MainActivity.this, getString(R.string.mvm_pet_deleted_success), Toast.LENGTH_SHORT).show();
                }
            }
        });

        // Observe deletePet operation error LiveData
        mainViewModel.getDeletePetError().observe(this, new Observer<String>() {
            @Override
            public void onChanged(String error) {
                if (error != null) {
                    Log.d(TAG, "Error deleting pet data: " + error);
                    mainViewModel.resetDeletePetError(); // Reset Livedata value
                    Toast.makeText(MainActivity.this, getString(R.string.mvm_pet_deleted_error), Toast.LENGTH_SHORT).show();
                }
            }
        });

        // Observe updatePet operation success LiveData
        mainViewModel.getUpdatePetSuccess().observe(this, new Observer<Boolean>() {
            @Override
            public void onChanged(Boolean success) {
                if (success != null && success) {
                    Log.d(TAG, "Pet data updated");
                    mainViewModel.resetUpdatePetSuccess(); // Reset Livedata value
                    Toast.makeText(MainActivity.this, getString(R.string.mvm_pet_updated_success), Toast.LENGTH_SHORT).show();
                }
            }
        });

        // Observe updatePet operation error LiveData
        mainViewModel.getUpdatePetError().observe(this, new Observer<String>() {
            @Override
            public void onChanged(String error) {
                if (error != null) {
                    Log.d(TAG, "Error updating pet data: " + error);
                    mainViewModel.resetUpdatePetError(); // Reset Livedata value
                    Toast.makeText(MainActivity.this, getString(R.string.mvm_pet_updated_error), Toast.LENGTH_SHORT).show();
                }
            }
        });

        // Observe addPost operation success LiveData
        mainViewModel.getAddPostSuccess().observe(this, new Observer<Boolean>() {
            @Override
            public void onChanged(Boolean success) {
                if (success != null && success) {
                    Log.d(TAG, "New post data saved at database");
                    mainViewModel.resetAddPostSuccess(); // Reset Livedata value
                    Toast.makeText(MainActivity.this, getString(R.string.mvm_post_added_success), Toast.LENGTH_SHORT).show();

                }
            }
        });

        // Observe addPost operation error LiveData
        mainViewModel.getAddPostError().observe(this, new Observer<String>() {
            @Override
            public void onChanged(String error) {
                if (error != null) {
                    Log.d(TAG,"Error adding post: " + error);
                    mainViewModel.resetAddPostError(); // Reset Livedata value
                    Toast.makeText(MainActivity.this, getString(R.string.mvm_post_added_error), Toast.LENGTH_SHORT).show();
                }
            }
        });

        // Observe addProduct operation success LiveData
        mainViewModel.getAddProductSuccess().observe(this, new Observer<Boolean>() {
            @Override
            public void onChanged(Boolean success) {
                if (success != null && success) {
                    Log.d(TAG, "New product data saved at database");
                    mainViewModel.resetAddProductSuccess(); // Reset Livedata value
                    Toast.makeText(MainActivity.this, getString(R.string.mvm_product_added_success), Toast.LENGTH_SHORT).show();
                }
            }
        });

        // Observe addProduct operation error LiveData
        mainViewModel.getAddProductError().observe(this, new Observer<String>() {
            @Override
            public void onChanged(String error) {
                if (error != null) {
                    Log.d(TAG, "Error adding product: " + error);
                    mainViewModel.resetAddProductError(); // Reset Livedata value
                    Toast.makeText(MainActivity.this, getString(R.string.mvm_product_added_error), Toast.LENGTH_SHORT).show();
                }
            }
        });

        // Observe deleteProduct operation success LiveData
        mainViewModel.getDeleteProductSuccess().observe(this, new Observer<Boolean>() {
            @Override
            public void onChanged(Boolean success) {
                if (success != null && success) {
                    Log.d(TAG, "Product deleted at database");
                    mainViewModel.resetDeleteProductSuccess(); // Reset Livedata value
                    Toast.makeText(MainActivity.this, getString(R.string.mvm_product_deleted_success), Toast.LENGTH_SHORT).show();
                }
            }
        });

        // Observe deleteProduct operation error LiveData
        mainViewModel.getDeleteProductError().observe(this, new Observer<String>() {
            @Override
            public void onChanged(String error) {
                if (error != null) {
                    Log.d(TAG, "Error deleting product: " + error);
                    mainViewModel.resetDeleteProductError(); // Reset Livedata value
                    Toast.makeText(MainActivity.this, getString(R.string.mvm_product_deleted_error), Toast.LENGTH_SHORT).show();
                }
            }
        });

        // Observe updateUserName operation success LiveData
        mainViewModel.getUpdateUserNameSuccess().observe(this, new Observer<Boolean>() {
            @Override
            public void onChanged(Boolean success) {
                if (success != null && success) {
                    Log.d(TAG, "User name updated");
                    mainViewModel.resetUpdateUserNameSuccess(); // Reset Livedata value
                    Toast.makeText(MainActivity.this, getString(R.string.mvm_user_name_updated_success), Toast.LENGTH_SHORT).show();
                }
            }
        });

        // Observe updateUserName operation error LiveData
        mainViewModel.getUpdateUserNameError().observe(this, new Observer<String>() {
            @Override
            public void onChanged(String error) {
                if (error != null) {
                    Log.d(TAG, "Error updating user name: " + error);
                    mainViewModel.resetUpdateUserNameError(); // Reset Livedata value
                    Toast.makeText(MainActivity.this, getString(R.string.mvm_user_name_updated_error), Toast.LENGTH_SHORT).show();
                }
            }
        });

        // Observe updateUserEmail operation success LiveData
        mainViewModel.getUpdateUserEmailSuccess().observe(this, new Observer<Boolean>() {
            @Override
            public void onChanged(Boolean success) {
                if (success != null && success) {
                    Log.d(TAG, "User email updated");
                    mainViewModel.resetUpdateUserEmailSuccess(); // Reset Livedata value
                    Toast.makeText(MainActivity.this, getString(R.string.mvm_user_email_updated_success), Toast.LENGTH_SHORT).show();
                }
            }
        });

        // Observe updateUserEmail operation error LiveData
        mainViewModel.getUpdateUserEmailError().observe(this, new Observer<String>() {
            @Override
            public void onChanged(String error) {
                if (error != null) {
                    Log.d(TAG, "Error updating user email: " + error);
                    mainViewModel.resetUpdateUserEmailError(); // Reset Livedata value
                    Toast.makeText(MainActivity.this, getString(R.string.mvm_user_email_updated_error), Toast.LENGTH_SHORT).show();
                }
            }
        });

        // Observe updateUserPassword operation success LiveData
        mainViewModel.getUpdateUserPasswordSuccess().observe(this, new Observer<Boolean>() {
            @Override
            public void onChanged(Boolean success) {
                if (success != null && success) {
                    Log.d(TAG, "User password updated");
                    mainViewModel.resetUpdateUserPasswordSuccess(); // Reset Livedata value
                    Toast.makeText(MainActivity.this, getString(R.string.mvm_user_password_updated_success), Toast.LENGTH_SHORT).show();
                }
            }
        });

        // Observe updateUserPassword operation error LiveData
        mainViewModel.getUpdateUserPasswordError().observe(this, new Observer<String>() {
            @Override
            public void onChanged(String error) {
                if (error != null) {
                    Log.d(TAG, "Error updating user password: " + error);
                    mainViewModel.resetUpdateUserPasswordError(); // Reset Livedata value
                    Toast.makeText(MainActivity.this, getString(R.string.mvm_user_password_updated_error), Toast.LENGTH_SHORT).show();
                }
            }
        });

        // Observe updateUserLanguage operation success LiveData
        mainViewModel.getUpdateUserLanguageSuccess().observe(this, new Observer<Boolean>() {
            @Override
            public void onChanged(Boolean success) {
                if (success != null && success) {
                    Log.d(TAG, "User language updated");
                    mainViewModel.resetUpdateUserLanguageSuccess(); // Reset Livedata value
                    Toast.makeText(MainActivity.this, getString(R.string.mvm_user_language_updated_success), Toast.LENGTH_SHORT).show();
                    recreate();
                }
            }
        });

        // Observe updateUserLanguage operation error LiveData
        mainViewModel.getUpdateUserLanguageError().observe(this, new Observer<String>() {
            @Override
            public void onChanged(String error) {
                if (error != null) {
                    Log.d(TAG, "Error updating user language: " + error);
                    mainViewModel.resetUpdateUserLanguageError(); // Reset Livedata value
                    Toast.makeText(MainActivity.this, getString(R.string.mvm_user_language_updated_error), Toast.LENGTH_SHORT).show();
                }
            }
        });

        // Observe logout operation success LiveData
        mainViewModel.getLogoutSuccess().observe(this, new Observer<Boolean>() {
            @Override
            public void onChanged(Boolean success) {
                if (success != null && success) {
                    Log.d(TAG, "User logout successfully");
                    mainViewModel.resetLogoutSuccess(); // Reset Livedata value
                    mainViewModel.clearUserIdFromPreferences(); // Clear userId from preferences

                    String userName = Objects.requireNonNull(mainViewModel.getCurrentUser().getValue()).getUserName();
                    LogoutByeDialogFragment logoutByeDialogFragment = LogoutByeDialogFragment.newInstance(userName);
                    logoutByeDialogFragment.show(getSupportFragmentManager(), "bye");

                    new Handler().postDelayed(new Runnable() {
                        @Override
                        public void run() {
                            Fragment fm = getSupportFragmentManager().findFragmentByTag("bye");
                            if (fm != null) getSupportFragmentManager().beginTransaction().remove(fm).commit();
                            // Go to LoginActivity
                            Intent intent = new Intent(getApplicationContext(), LoginActivity.class);
                            startActivity(intent);
                            finish();
                        }
                    }, SPLASH_DIALOG);
                }
            }
        });

        // Observe logout operation error LiveData
        mainViewModel.getLogoutError().observe(this, new Observer<String>() {
            @Override
            public void onChanged(String error) {
                if (error != null) {
                    Log.d(TAG, "Error during user logout: " + error);
                    mainViewModel.resetLogoutError(); // Reset Livedata value
                    Toast.makeText(MainActivity.this, getString(R.string.mvm_user_logout_error), Toast.LENGTH_SHORT).show();
                }
            }
        });
    }
}