package edu.uoc.tfg.yourlovelypet.views.adapters;

import android.content.Context;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.TextView;
import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.PopupMenu;
import androidx.navigation.NavController;
import androidx.navigation.Navigation;
import androidx.recyclerview.widget.RecyclerView;
import com.bumptech.glide.Glide;
import com.bumptech.glide.load.resource.bitmap.CenterCrop;
import com.bumptech.glide.load.resource.bitmap.RoundedCorners;
import com.bumptech.glide.request.RequestOptions;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import edu.uoc.tfg.yourlovelypet.R;
import edu.uoc.tfg.yourlovelypet.databinding.PetRowBinding;
import edu.uoc.tfg.yourlovelypet.model.Pet;
import edu.uoc.tfg.yourlovelypet.repository.FirestorageRepository;
import edu.uoc.tfg.yourlovelypet.viewModel.MainViewModel;
import edu.uoc.tfg.yourlovelypet.views.dialogs.ConfirmPetDeleteDialogFragment;
import edu.uoc.tfg.yourlovelypet.views.dialogs.EditPetDialogFragment;

public class PetAdapter extends RecyclerView.Adapter<PetAdapter.ViewHolder> {

    private static final String TAG = "[PetAdapter]";
    private static final String DEFAULT = "default";
    private MainViewModel mainViewModel;
    private Context context;
    private List<Pet> dataSet = new ArrayList<>(); // List of pets to send

    public PetAdapter(MainViewModel mainViewModel, Context context) {
        this.mainViewModel = mainViewModel;
        this.context = context;
    }

    public class ViewHolder extends RecyclerView.ViewHolder implements View.OnClickListener, PopupMenu.OnMenuItemClickListener {

        private PetRowBinding recyclerRowBinding;
        private final TextView tvPetName;
        private final TextView tvPetAnimal;
        private final ImageButton menu;
        private final TextView tvPetBirthdate;
        private final ImageView ivPetDefaultImage;
        private PopupMenu popupMenu;
        private final ImageButton linkToGallery;
        private final ImageView linkToCalendar;
        private final ImageView linkToPetProducts;
        private final ImageView linkToHealth;

        public ViewHolder(@NonNull PetRowBinding recyclerRowBinding) {
            super(recyclerRowBinding.getRoot());
            this.recyclerRowBinding = recyclerRowBinding;

            tvPetName = recyclerRowBinding.tvPetName;
            tvPetAnimal = recyclerRowBinding.tvPetAnimal;
            tvPetBirthdate = recyclerRowBinding.tvPetBirthdate;
            ivPetDefaultImage = recyclerRowBinding.petDefaultImage;
            menu = recyclerRowBinding.tvPetMenu;
            menu.setOnClickListener(this);
            linkToGallery = recyclerRowBinding.toGallery;
            linkToCalendar = recyclerRowBinding.toCalendar;
            linkToPetProducts = recyclerRowBinding.toPetProducts;
            linkToHealth = recyclerRowBinding.toHealth;

            // Link to pet fragment gallery
            linkToGallery.setOnClickListener(v -> {
                int position = getBindingAdapterPosition();
                if (position != RecyclerView.NO_POSITION) {
                    Bundle bundle = new Bundle();
                    bundle.putString("petId",dataSet.get(position).getId());
                    Log.d(TAG,"Sending petId: " + dataSet.get(position).getId() + " to gallery fragment");
                    NavController navController = Navigation.findNavController((AppCompatActivity) context, R.id.main);
                    navController.navigate(R.id.action_homeFragment_to_galleryFragment, bundle);
                }
            });

            // Link to pet fragment calendar
            linkToCalendar.setOnClickListener(v -> {
                int position = getBindingAdapterPosition();
                if (position != RecyclerView.NO_POSITION) {
                    Bundle bundle = new Bundle();
                    bundle.putString("petId",dataSet.get(position).getId());
                    NavController navController = Navigation.findNavController((AppCompatActivity) context, R.id.main);
                    navController.navigate(R.id.action_homeFragment_to_calendarFragment, bundle);
                }
            });

            // Link to pet fragment petProducts
            linkToPetProducts.setOnClickListener(v -> {
                int position = getBindingAdapterPosition();
                if (position != RecyclerView.NO_POSITION) {
                    Bundle bundle = new Bundle();
                    bundle.putString("petId",dataSet.get(position).getId());
                    NavController navController = Navigation.findNavController((AppCompatActivity) context, R.id.main);
                    navController.navigate(R.id.action_homeFragment_to_petProductFragment, bundle);
                }
            });

            // Link to pet fragment health
            linkToHealth.setOnClickListener(v -> {
                int position = getBindingAdapterPosition();
                if (position != RecyclerView.NO_POSITION) {
                    Bundle bundle = new Bundle();
                    bundle.putString("petId",dataSet.get(position).getId());
                    NavController navController = Navigation.findNavController((AppCompatActivity) context, R.id.main);
                    navController.navigate(R.id.action_homeFragment_to_healthFragment, bundle);
                }
            });
        }

        public TextView getTvPetName() {
            return tvPetName;
        }
        public TextView getTvPetAnimal() {
            return tvPetAnimal;
        }
        public TextView getTvPetBirthdate() {
            return tvPetBirthdate;
        }
        public ImageView getIvPetDefaultImage() {
            return ivPetDefaultImage;
        }

        @Override
        public void onClick(View view) {
            showMenu(view);
        }

        private void showMenu(View view) {
            popupMenu = new PopupMenu(view.getContext(), view);
            popupMenu.inflate(R.menu.pet_menu);
            popupMenu.setOnMenuItemClickListener(this);
            popupMenu.show();
        }

        @Override
        public boolean onMenuItemClick(MenuItem item) {
            int position = getBindingAdapterPosition();

            if (position != RecyclerView.NO_POSITION) {
                Pet petToEdit = dataSet.get(position);
                String petId = petToEdit.getId();

                if (item.getItemId() == R.id.edit) {
                    Log.d(TAG, "Clicked option edit at menu over pet with Id: " + petId + " its name is: " + petToEdit.getPetName());
                    EditPetDialogFragment editPetDialogFragment = EditPetDialogFragment.newInstance(petId);
                    editPetDialogFragment.show(((AppCompatActivity) context).getSupportFragmentManager(), "editPet");
                    return true;
                } else if (item.getItemId() == R.id.delete) {
                    Log.d(TAG, "Clicked option delete at menu over pet with Id: " + petId + " its name is: " + petToEdit.getPetName());
                    ConfirmPetDeleteDialogFragment confirmPetDeleteDialogFragment = ConfirmPetDeleteDialogFragment.newInstance(petId);
                    confirmPetDeleteDialogFragment.show(((AppCompatActivity) context).getSupportFragmentManager(), "confirmDeletePet");
                    return true;
                } else if (item.getItemId() == R.id.image_profile) {
                    Log.d(TAG, "Clicked option image_profile at menu over pet with Id: " + dataSet.get(position).getId() + " its name is: " + dataSet.get(position).getPetName());
                    Bundle bundle = new Bundle();
                    bundle.putString("petId", petId);
                    NavController navController = Navigation.findNavController((AppCompatActivity) context, R.id.main);
                    navController.navigate(R.id.action_homeFragment_to_galleryFragment, bundle);
                    return true;
                } else {
                    return false;
                }
            }
            return false;
        }
    }

    public void setData(List<Pet> pets) {
        this.dataSet = pets;
        this.notifyDataSetChanged(); // ViewModel: notify to observers
    }

    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(@NonNull ViewGroup viewGroup, int viewType) {
        LayoutInflater layoutInflater = LayoutInflater.from(viewGroup.getContext());
        PetRowBinding recyclerRowBinding = PetRowBinding.inflate(layoutInflater, viewGroup, false);
        return new ViewHolder(recyclerRowBinding);
    }

    @Override
    public void onBindViewHolder(@NonNull ViewHolder viewHolder, final int position) {
        Pet pet = dataSet.get(position);
        viewHolder.getTvPetName().setText(pet.getPetName());
        viewHolder.getTvPetAnimal().setText(pet.getPetAnimal());
        viewHolder.getTvPetBirthdate().setText(pet.getPetBirthdate());
        String profileImage = pet.getImageProfile();

        // Default pet image based on pet type
        int defaultId;
        switch (pet.getPetAnimal()) {
            case "Dog":
                defaultId = R.drawable.default_dog;
                break;
            case "Cat":
                defaultId = R.drawable.default_cat;
                break;
            case "Bird":
                defaultId = R.drawable.default_bird;
                break;
            case "Fish":
                defaultId = R.drawable.default_fish;
                break;
            default:
                defaultId = R.drawable.default_other;
                break;
        }
        viewHolder.getIvPetDefaultImage().setImageResource(defaultId);

        if (profileImage != null && !profileImage.equals(DEFAULT)) {

            String userId = Objects.requireNonNull(mainViewModel.getCurrentUser().getValue()).getId();
            String petId = pet.getId();
            String path = userId + "/" + petId + "/" + profileImage;

            mainViewModel.getUrl(path, new FirestorageRepository.Callback<Uri>() {
                @Override
                public void onSuccess(Uri result) {
                    if (result != null) {

                        // Glide: define transformation
                        RequestOptions requestOptions = new RequestOptions()
                                .placeholder(defaultId)
                                .transform(new CenterCrop(), new RoundedCorners(20));

                        // Glide: load image applying transformation
                        Glide.with(context).load(result).apply(requestOptions)
                                .into(viewHolder.getIvPetDefaultImage());
                    }
                }

                @Override
                public void onFailure(String error) {
                    Log.e(TAG,"Error downloading profile image of pet: " + petId + " : " + error);
                }
            });
        }
    }

    @Override
    public int getItemCount() {
        return dataSet.size();
    }
}