package edu.uoc.tfg.yourlovelypet.views.adapters;

import android.content.Context;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.navigation.NavController;
import androidx.navigation.Navigation;
import androidx.recyclerview.widget.RecyclerView;
import com.bumptech.glide.Glide;
import java.util.ArrayList;
import java.util.List;
import edu.uoc.tfg.yourlovelypet.R;
import edu.uoc.tfg.yourlovelypet.databinding.PhotoRowBinding;
import edu.uoc.tfg.yourlovelypet.viewModel.MainViewModel;

public class PhotoAdapter extends RecyclerView.Adapter<PhotoAdapter.ViewHolder> {

    private static final String TAG = "[PhotoAdapter]";
    private static final String ARG1 = "imageUrl";
    private static final String ARG2 = "petId";
    private Context context;
    private String petId;
    private List<String> dataSet = new ArrayList<>(); // List of pet images references to send

    public PhotoAdapter(String petId, Context context) {
        this.context = context;
        this.petId = petId;
    }

    public class ViewHolder extends RecyclerView.ViewHolder implements View.OnClickListener {

        private ImageView ivPhoto;
        private PhotoRowBinding recyclerRowBinding;

        public ViewHolder(@NonNull PhotoRowBinding recyclerRowBinding) {
            super(recyclerRowBinding.getRoot());
            this.recyclerRowBinding = recyclerRowBinding;
            ivPhoto = recyclerRowBinding.imageView;
            ivPhoto.setOnClickListener(this);
        }

        public ImageView getIvPhoto() {
            return ivPhoto;
        }

        @Override
        public void onClick(View view) {
            int position = getBindingAdapterPosition();
            if (position != RecyclerView.NO_POSITION) {
                String imageUrl = dataSet.get(position);
                Bundle bundle = new Bundle();
                bundle.putString(ARG1, imageUrl);
                bundle.putString(ARG2, petId);

                Log.d(TAG,"Go to photo fragment. Click detected over image: " + imageUrl);
                NavController navController = Navigation.findNavController((AppCompatActivity) context, R.id.main);
                navController.navigate(R.id.action_galleryFragment_to_photoFragment, bundle);
            }
        }
    }

    public void setData(List<String> photos) {
        this.dataSet = photos;
        this.notifyDataSetChanged(); // ViewModel: notify to observers
    }

    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(@NonNull ViewGroup viewGroup, int viewType) {
        LayoutInflater layoutInflater = LayoutInflater.from(viewGroup.getContext());
        PhotoRowBinding recyclerRowBinding = PhotoRowBinding.inflate(layoutInflater, viewGroup, false);
        return new ViewHolder(recyclerRowBinding);
    }

    @Override
    public void onBindViewHolder(@NonNull ViewHolder viewHolder, int position) {
        String imageUrl = dataSet.get(position);
        Glide.with(context).load(imageUrl).into(viewHolder.getIvPhoto());
    }

    @Override
    public int getItemCount() {
        return dataSet.size();
    }
}
