package edu.uoc.tfg.yourlovelypet.views.adapters;

import android.content.Context;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageButton;
import android.widget.TextView;
import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.navigation.fragment.NavHostFragment;
import androidx.recyclerview.widget.RecyclerView;
import java.util.ArrayList;
import java.util.List;
import edu.uoc.tfg.yourlovelypet.R;
import edu.uoc.tfg.yourlovelypet.databinding.PostRowBinding;
import edu.uoc.tfg.yourlovelypet.model.Post;
import edu.uoc.tfg.yourlovelypet.viewModel.MainViewModel;
import edu.uoc.tfg.yourlovelypet.views.dialogs.ReplyPostDialogFragment;
import edu.uoc.tfg.yourlovelypet.views.fragments.BoardFragment;

public class PostAdapter extends RecyclerView.Adapter<PostAdapter.ViewHolder>  {

    private static final String TAG = "[PostAdapter]";
    private MainViewModel mainViewModel;
    private Context context;
    private List<Post> dataSet = new ArrayList<>(); // List of posts to send

    public PostAdapter(MainViewModel mainViewModel, Context context) {
        this.mainViewModel = mainViewModel;
        this.context = context;
    }
    public class ViewHolder extends RecyclerView.ViewHolder {

        private PostRowBinding recyclerRowBinding;
        private final TextView postTitle;
        private final TextView postAuthor;
        private final TextView postDate;
        private final TextView postBody;
        private final TextView postReplies;
        private final ImageButton reply;
        private String postId = "";

        public ViewHolder(@NonNull PostRowBinding recyclerRowBinding) {
            super(recyclerRowBinding.getRoot());
            this.recyclerRowBinding = recyclerRowBinding;

            postTitle = recyclerRowBinding.postTitle;
            postAuthor = recyclerRowBinding.postAuthor;
            postDate = recyclerRowBinding.postDate;
            postBody = recyclerRowBinding.postBody;
            postReplies = recyclerRowBinding.replies;
            reply = recyclerRowBinding.postReply;

            reply.setOnClickListener(v -> {
                int position = getBindingAdapterPosition();
                if (position != RecyclerView.NO_POSITION) {
                    postId = dataSet.get(position).getId();
                    Log.d(TAG,"Sending postId: " + postId + " to new reply fragment");
                    ReplyPostDialogFragment replyPostDialogFragment = ReplyPostDialogFragment.newInstance(postId);
                    replyPostDialogFragment.show(((AppCompatActivity) context).getSupportFragmentManager(), "replyPost");
                }
            });

            postReplies.setOnClickListener(v -> {
                int position = getBindingAdapterPosition();
                if (position != RecyclerView.NO_POSITION) {
                    postId = dataSet.get(position).getId();
                    Log.d(TAG,"Request to show replies to post with id: " + postId);
                    if (context instanceof AppCompatActivity) {
                        NavHostFragment navHostFragment = (NavHostFragment) ((AppCompatActivity) context).getSupportFragmentManager().findFragmentById(R.id.main);
                        BoardFragment boardFragment = (BoardFragment) navHostFragment.getChildFragmentManager().getFragments().get(0);
                        if (boardFragment != null) {
                            boardFragment.showReplies(postId);
                        }
                    }
                }
            });
        }

        public TextView getPostTitle() {
            return postTitle;
        }

        public TextView getPostAuthor() {
            return postAuthor;
        }
        public TextView getPostDate() {
            return postDate;
        }
        public TextView getPostBody() {
            return postBody;
        }
    }

    public void setData(List<Post> posts) {
        this.dataSet = posts;
        this.notifyDataSetChanged(); // ViewModel: notify to observers
    }

    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(@NonNull ViewGroup viewGroup, int viewType) {
        LayoutInflater layoutInflater = LayoutInflater.from(viewGroup.getContext());
        PostRowBinding recyclerRowBinding = PostRowBinding.inflate(layoutInflater, viewGroup, false);
        return new ViewHolder(recyclerRowBinding);
    }

    @Override
    public void onBindViewHolder(@NonNull ViewHolder viewHolder, final int position) {
        Post currentPosition = dataSet.get(position);
        viewHolder.getPostTitle().setText(currentPosition.getTitle());
        viewHolder.getPostAuthor().setText(currentPosition.getAuthor());
        viewHolder.getPostDate().setText(currentPosition.getDate());
        viewHolder.getPostBody().setText(currentPosition.getBody());

        int replies = mainViewModel.getRepliesNumber(currentPosition.getId());
        if (replies != 0) {
            String repliesString = context.getString(R.string.post_count_replies, replies);
            viewHolder.postReplies.setText(repliesString);
            viewHolder.postReplies.setVisibility(View.VISIBLE);
        } else {
            viewHolder.postReplies.setVisibility(View.GONE);
        }
    }

    @Override
    public int getItemCount() {
        return dataSet.size();
    }
}