package edu.uoc.tfg.yourlovelypet.views.dialogs;

import android.app.DatePickerDialog;
import android.app.Dialog;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;
import androidx.lifecycle.ViewModelProvider;
import android.text.TextUtils;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AutoCompleteTextView;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Locale;
import edu.uoc.tfg.yourlovelypet.R;
import edu.uoc.tfg.yourlovelypet.databinding.FragmentAddPetDialogBinding;
import edu.uoc.tfg.yourlovelypet.model.Pet;
import edu.uoc.tfg.yourlovelypet.viewModel.MainViewModel;

public class AddPetDialogFragment extends DialogFragment {

    private static final String TAG = "[AddPet]";
    private FragmentAddPetDialogBinding binding;
    private MainViewModel mainViewModel;
    private AutoCompleteTextView autoCompletePetType;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @NonNull
    @Override
    public Dialog onCreateDialog(@Nullable Bundle savedInstanceState) {
        Dialog dialog = super.onCreateDialog(savedInstanceState);
        if (dialog.getWindow() != null) {
            dialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
            dialog.getWindow().setBackgroundDrawableResource(R.drawable.custom_bg_heaven);
        }
        return dialog;
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        binding = FragmentAddPetDialogBinding.inflate(getLayoutInflater(), container, false);
        mainViewModel = new ViewModelProvider(requireActivity()).get(MainViewModel.class);
        autoCompletePetType = binding.editPetAnimal;
        return binding.getRoot();
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        binding.editPetAnimal.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                autoCompletePetType.showDropDown();
            }
        });

        binding.editPetBirthdate.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                showDatePickerDialog();
            }
        });

        binding.buttonAdd.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                String petName = binding.editPetName.getEditableText().toString();
                String petAnimal = binding.editPetAnimal.getEditableText().toString();
                String petBreed = binding.editPetBreed.getEditableText().toString();
                String petWeight = binding.editPetWeight.getEditableText().toString();
                String petBirthdate = binding.editPetBirthdate.getEditableText().toString();
                // Convert to english in case of petAnimal spanish input
                String englishPetAnimal = convertToEnglish(petAnimal);

                if (validateFields(petName, englishPetAnimal, petBreed, petWeight, petBirthdate)) {
                    Pet newPet = new Pet(petName, englishPetAnimal, petBreed, petWeight, petBirthdate);
                    Log.d(TAG, "Request to add a new pet with name: " + petName);
                    mainViewModel.addPet(newPet);
                    dismiss();
                }
            }
        });

        binding.buttonCancel.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                dismiss();
            }
        });
    }

    private boolean validateFields(String petName, String petAnimal, String petBreed, String petWeight, String petBirthdate) {

        if (TextUtils.isEmpty(petName)) {
            binding.editPetName.setError(getString(R.string.addPet_error_pet_name_required));
            return false;
        }

        if (TextUtils.isEmpty(petAnimal)) {
            binding.editPetAnimal.setError(getString(R.string.addPet_error_pet_animal_required));
            return false;
        }

        if (TextUtils.isEmpty(petBreed)) {
            binding.editPetBreed.setError(getString(R.string.addPet_error_pet_breed_required));
            return false;
        }

        if (TextUtils.isEmpty(petWeight)) {
            binding.editPetWeight.setError(getString(R.string.addPet_error_pet_weight_required));
            return false;
        } else {
            try {
                Double.parseDouble(petWeight);
            } catch (NumberFormatException e) {
                binding.editPetWeight.setError(getString(R.string.addPet_error_pet_weight_invalid));
                return false;
            }
        }

        if (TextUtils.isEmpty(petBirthdate)) {
            binding.editPetBirthdate.setError(getString(R.string.addPet_error_pet_birthdate_required));
            return false;
        }

        return true;
    }

    private String convertToEnglish(String bilingualPetAnimal) {
        switch (bilingualPetAnimal) {
            case "Perro":
            case "Dog":
                return "Dog";
            case "Gato":
            case "Cat":
                return "Cat";
            case "Pájaro":
            case "Bird":
                return "Bird";
            case "Pez":
            case "Fish":
                return "Fish";
            default:
                return "Other";
        }
    }

    private void showDatePickerDialog() {

        Calendar calendar = Calendar.getInstance();
        int year = calendar.get(Calendar.YEAR);
        int month = calendar.get(Calendar.MONTH);
        int dayOfMonth = calendar.get(Calendar.DAY_OF_MONTH);

        DatePickerDialog datePickerDialog = new DatePickerDialog(
                requireContext(),
                R.style.CustomDatePickerDialogTheme,
                (view, year1, month1, dayOfMonth1) -> {
                    Calendar selectedDate = Calendar.getInstance();
                    selectedDate.set(year1,month1,dayOfMonth1);
                    SimpleDateFormat dateFormat = new SimpleDateFormat("dd/MM/yyyy", Locale.getDefault());
                    String selectedDateString = dateFormat.format(selectedDate.getTime());
                    binding.editPetBirthdate.setText(selectedDateString);
                },
                year, month, dayOfMonth
        );
        datePickerDialog.show();
    }
}