package edu.uoc.tfg.yourlovelypet.views.dialogs;

import android.app.Dialog;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Toast;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.DialogFragment;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModelProvider;
import androidx.navigation.NavController;
import androidx.navigation.Navigation;
import edu.uoc.tfg.yourlovelypet.R;
import edu.uoc.tfg.yourlovelypet.databinding.FragmentConfirmPhotoDeleteDialogBinding;
import edu.uoc.tfg.yourlovelypet.viewModel.MainViewModel;

public class ConfirmPhotoDeleteDialogFragment extends DialogFragment {
    private static final String TAG = "[ConfirmPhotoDelete]";
    private FragmentConfirmPhotoDeleteDialogBinding binding;
    private MainViewModel mainViewModel;
    private static final String ARG1 = "imageUrl";
    private static final String ARG2 = "petId";
    private String petId;
    private String imageUrl;

    public ConfirmPhotoDeleteDialogFragment() {}

    public static ConfirmPhotoDeleteDialogFragment newInstance(String param1, String param2) {
        ConfirmPhotoDeleteDialogFragment fragment = new ConfirmPhotoDeleteDialogFragment();
        Bundle args = new Bundle();
        args.putString(ARG1, param1);
        args.putString(ARG2, param2);
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (getArguments() != null) {
            imageUrl = getArguments().getString(ARG1);
            petId = getArguments().getString(ARG2);
            Log.d(TAG,"Receive imageUrl: " + imageUrl);
            Log.d(TAG,"Receive petId: " + petId);
        }
    }

    @NonNull
    @Override
    public Dialog onCreateDialog(@Nullable Bundle savedInstanceState) {
        Dialog dialog = super.onCreateDialog(savedInstanceState);
        if (dialog.getWindow() != null) {
            dialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
            dialog.getWindow().setBackgroundDrawableResource(R.drawable.custom_bg_white);
        }
        return dialog;
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        binding = FragmentConfirmPhotoDeleteDialogBinding.inflate(getLayoutInflater(), container, false);
        mainViewModel = new ViewModelProvider(requireActivity()).get(MainViewModel.class);
        return binding.getRoot();
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        // Observe deleteImage operation success LiveData
        mainViewModel.getDeleteImageSuccess().observe(getViewLifecycleOwner(), new Observer<Boolean>() {
            @Override
            public void onChanged(Boolean success) {
                if (success != null && success) {
                    Log.d(TAG,"Image deleted from firestorage and firestore");
                    mainViewModel.resetDeleteImageSuccess(); // Reset Livedata value
                    Toast.makeText(getContext(),"Image deleted", Toast.LENGTH_SHORT).show(); // TODO replace with string resource
                    // Remove PhotoFragment from back stack
                    NavController navController = Navigation.findNavController((AppCompatActivity) requireContext(), R.id.main);
                    navController.popBackStack(R.id.photoFragment, true);
                    // Navigate to GalleryFragment
                    Bundle bundle = new Bundle();
                    bundle.putString("petId", petId);
                    navController.navigate(R.id.galleryFragment, bundle);
                    dismiss();
                }
            }
        });

        // Observe deleteImage operation error LiveData
        mainViewModel.getDeleteImageError().observe(getViewLifecycleOwner(), new Observer<String>() {
            @Override
            public void onChanged(String error) {
                if (error != null) {
                    Log.d(TAG,"Error deleting image: " + error);
                    mainViewModel.resetDeleteImageError(); // Reset Livedata value
                    Toast.makeText(getContext(), "Error deleting image, please try again", Toast.LENGTH_SHORT).show(); // TODO replace with string resource
                }
            }
        });

        binding.buttonYes.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Log.d(TAG,"Request to delete image with url: " + imageUrl);
                mainViewModel.deleteImage(petId, imageUrl);
            }
        });

        binding.buttonNo.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                dismiss();
            }
        });
    }
}
