package edu.uoc.tfg.yourlovelypet.views.dialogs;

import android.app.Dialog;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;
import androidx.lifecycle.ViewModelProvider;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Locale;
import edu.uoc.tfg.yourlovelypet.R;
import edu.uoc.tfg.yourlovelypet.databinding.FragmentNewPostDialogBinding;
import edu.uoc.tfg.yourlovelypet.model.Post;
import edu.uoc.tfg.yourlovelypet.viewModel.MainViewModel;

public class NewPostDialogFragment extends DialogFragment {

    private static final String TAG = "[NewPost]";
    private static final String ROOT = "root";
    private FragmentNewPostDialogBinding binding;
    private MainViewModel mainViewModel;

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @NonNull
    @Override
    public Dialog onCreateDialog(@Nullable Bundle savedInstanceState) {
        Dialog dialog = super.onCreateDialog(savedInstanceState);
        if (dialog.getWindow() != null) {
            dialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
            dialog.getWindow().setBackgroundDrawableResource(R.drawable.custom_bg_white);
        }
        return dialog;
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        binding = FragmentNewPostDialogBinding.inflate(getLayoutInflater(), container, false);
        mainViewModel = new ViewModelProvider(requireActivity()).get(MainViewModel.class);
        return binding.getRoot();
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        binding.send.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                String postTitle = binding.editPostTitle.getEditableText().toString();
                String postBody = binding.editPostBody.getEditableText().toString();
                Log.d(TAG, "Request to add new post");

                if (validateFields(postTitle, postBody)) {
                    String author = mainViewModel.getCurrentUser().getValue().getUserName();

                    Calendar calendar = Calendar.getInstance();
                    SimpleDateFormat simpleDateFormat = new SimpleDateFormat("dd/MM/yyyy", Locale.US);
                    String date = simpleDateFormat.format(calendar.getTime());

                    Post post = new Post(postTitle, author, date, postBody, ROOT);
                    mainViewModel.addPost(post);
                    dismiss();
                }
            }
        });

        binding.delete.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                dismiss();
            }
        });
    }

    private boolean validateFields(String postTitle, String postBody) {

        if (TextUtils.isEmpty(postTitle)) {
            binding.editPostTitle.setError(getString(R.string.new_post_error_title_required));
            return false;
        }

        if (TextUtils.isEmpty(postBody)) {
            binding.editPostBody.setError(getString(R.string.new_post_error_body_required));
            return false;
        }

        return true;
    }
}
