package edu.uoc.tfg.yourlovelypet.views.dialogs;

import android.app.Dialog;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;
import androidx.lifecycle.ViewModelProvider;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Locale;
import edu.uoc.tfg.yourlovelypet.R;
import edu.uoc.tfg.yourlovelypet.databinding.FragmentReplyPostDialogBinding;
import edu.uoc.tfg.yourlovelypet.model.Post;
import edu.uoc.tfg.yourlovelypet.viewModel.MainViewModel;

public class ReplyPostDialogFragment extends DialogFragment {

    private static final String TAG = "[ReplyPost]";
    private static final String ARG_POST_ID = "postId";
    private String receivedPostId;
    private Post postToReply;
    private FragmentReplyPostDialogBinding binding;
    private MainViewModel mainViewModel;

    public ReplyPostDialogFragment() {}

    public static ReplyPostDialogFragment newInstance(String param) {
        ReplyPostDialogFragment fragment = new ReplyPostDialogFragment();
        Bundle args = new Bundle();
        args.putString(ARG_POST_ID, param);
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (getArguments() != null) {
            receivedPostId = getArguments().getString(ARG_POST_ID);
        }
    }

    @NonNull
    @Override
    public Dialog onCreateDialog(@Nullable Bundle savedInstanceState) {
        Dialog dialog = super.onCreateDialog(savedInstanceState);
        if (dialog.getWindow() != null) {
            dialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
            dialog.getWindow().setBackgroundDrawableResource(R.drawable.custom_bg_white);
        }
        return dialog;
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        binding = FragmentReplyPostDialogBinding.inflate(getLayoutInflater(), container, false);
        mainViewModel = new ViewModelProvider(requireActivity()).get(MainViewModel.class);
        return binding.getRoot();
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        Log.d(TAG,"Received postId: " + receivedPostId);
        postToReply = mainViewModel.getPostById(receivedPostId);
        binding.mainPostTitle.setText(postToReply.getTitle());

        binding.send.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                String postTitle = "Re: " + postToReply.getTitle();
                String postBody = binding.editPostBody.getEditableText().toString();

                Log.d(TAG, "Request to add new reply to post with id: " + receivedPostId);
                if (validateFields(postBody)) {
                    String author = mainViewModel.getCurrentUser().getValue().getUserName();
                    Calendar calendar = Calendar.getInstance();
                    SimpleDateFormat simpleDateFormat = new SimpleDateFormat("dd/MM/yyyy", Locale.US);
                    String date = simpleDateFormat.format(calendar.getTime());
                    Post post = new Post(postTitle, author, date, postBody, receivedPostId);
                    mainViewModel.addPost(post);
                    dismiss();
                }
            }
        });

        binding.delete.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                dismiss();
            }
        });
    }

    private boolean validateFields(String postBody) {

        if (TextUtils.isEmpty(postBody)) {
            binding.editPostBody.setError(getString(R.string.new_post_error_body_required));
            return false;
        }

        return true;
    }
}
