package edu.uoc.tfg.yourlovelypet.views.fragments;

import android.os.Bundle;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModelProvider;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import java.util.ArrayList;
import java.util.List;
import edu.uoc.tfg.yourlovelypet.databinding.FragmentBoardBinding;
import edu.uoc.tfg.yourlovelypet.model.Post;
import edu.uoc.tfg.yourlovelypet.model.User;
import edu.uoc.tfg.yourlovelypet.viewModel.MainViewModel;
import edu.uoc.tfg.yourlovelypet.views.adapters.PostAdapter;
import edu.uoc.tfg.yourlovelypet.views.dialogs.NewPostDialogFragment;

public class BoardFragment extends Fragment {

    private static final String TAG = "[BoardFragment]";
    private MainViewModel mainViewModel;
    private FragmentBoardBinding binding;
    private RecyclerView recyclerView;
    private PostAdapter postAdapter;
    private boolean isShowingReplies = false;
    private String parentPostId;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        binding = FragmentBoardBinding.inflate(getLayoutInflater(), container, false);
        return binding.getRoot();
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        mainViewModel = new ViewModelProvider(requireActivity()).get(MainViewModel.class);

        // RecyclerView y Adapter
        recyclerView = binding.recyclerViewListOfPosts;
        postAdapter = new PostAdapter(mainViewModel, getContext()); // Send context to show new dialogFragment from there
        recyclerView.setAdapter(postAdapter);
        recyclerView.setLayoutManager(new LinearLayoutManager(getContext()));

        // Observe currentUser like LiveData
        mainViewModel.getCurrentUser().observe(getViewLifecycleOwner(), new Observer<User>() {
            @Override
            public void onChanged(User user) {
                binding.textViewUserName.setText(user.getUserName());
            }
        });

        // Observe posts like LiveData
        mainViewModel.getPosts().observe(getViewLifecycleOwner(), new Observer<List<Post>>() {
            @Override
            public void onChanged(List<Post> posts) {
                if ( isShowingReplies && parentPostId != null) {
                    // Filter to show replies only
                    List<Post> replies = filterReplies(posts, parentPostId);
                    postAdapter.setData(replies);
                } else {
                    // Show only main posts (with field replyToId equal to "root")
                    List<Post> mainPosts = filterMainPosts(posts);
                    postAdapter.setData(mainPosts);
                }
            }
        });

        // Navigation to main posts
        binding.backButton.setOnClickListener(v -> {
            isShowingReplies = false;
            parentPostId = null;
            binding.backButton.setVisibility(View.GONE);
            mainViewModel.refreshPosts();
        });

        binding.addPostButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                NewPostDialogFragment newPostDialogFragment = new NewPostDialogFragment();
                newPostDialogFragment.show(getParentFragmentManager(),"addPost");
            }
        });
    }

    private List<Post> filterMainPosts(List<Post> posts) {
        List<Post> mainPosts = new ArrayList<>();
        for (Post post : posts) {
            if ("root".equals(post.getReplyToId())) {
                mainPosts.add(post);
            }
        }
        return mainPosts;
    }

    private List<Post> filterReplies(List<Post> posts, String parentId) {
        List<Post> replies = new ArrayList<>();
        for (Post post : posts) {
            if (parentId.equals(post.getReplyToId())) {
                replies.add(post);
            }
        }
        return replies;
    }

    public void showReplies(String parentId) {
        isShowingReplies = true;
        parentPostId = parentId;
        binding.backButton.setVisibility(View.VISIBLE);
        mainViewModel.refreshPosts();
    }
}