package edu.uoc.tfg.yourlovelypet.views.fragments;

import android.os.Bundle;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModelProvider;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import java.util.List;
import edu.uoc.tfg.yourlovelypet.views.adapters.PetAdapter;
import edu.uoc.tfg.yourlovelypet.databinding.FragmentHomeBinding;
import edu.uoc.tfg.yourlovelypet.model.Pet;
import edu.uoc.tfg.yourlovelypet.model.User;
import edu.uoc.tfg.yourlovelypet.viewModel.MainViewModel;
import edu.uoc.tfg.yourlovelypet.views.dialogs.AddPetDialogFragment;

public class HomeFragment extends Fragment {

    private static final String TAG = "[HomeFragment]";
    private FragmentHomeBinding binding;
    private MainViewModel mainViewModel;
    private RecyclerView recyclerView;
    private PetAdapter petAdapter;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        binding = FragmentHomeBinding.inflate(getLayoutInflater(), container, false);
        return binding.getRoot();
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        mainViewModel = new ViewModelProvider(requireActivity()).get(MainViewModel.class);

        // RecyclerView y Adapter
        recyclerView = binding.recyclerViewListOfPets;
        petAdapter = new PetAdapter(mainViewModel, getContext()); // Send context to show new dialogFragment from there
        recyclerView.setAdapter(petAdapter);
        recyclerView.setLayoutManager(new LinearLayoutManager(getContext()));

        // Observer of current user LiveData
        mainViewModel.getCurrentUser().observe(getViewLifecycleOwner(), new Observer<User>() {
            @Override
            public void onChanged(User user) {
                binding.textViewUserName.setText(user.getUserName());
            }
        });

        // Observer of pets from current user LiveData
        mainViewModel.getPetsFromCurrentUser().observe(getViewLifecycleOwner(), new Observer<List<Pet>>() {
            @Override
            public void onChanged(List<Pet> items) {
                if (items == null || items.isEmpty()) {
                    binding.textViewNoPets.setVisibility(View.VISIBLE);
                } else {
                    binding.textViewNoPets.setVisibility(View.GONE);
                }
                petAdapter.setData(items);
            }
        });

        binding.addPetButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                AddPetDialogFragment addPetDialogFragment = new AddPetDialogFragment();
                addPetDialogFragment.show(getParentFragmentManager(),"addPet");
            }
        });
    }
}