package edu.uoc.tfg.yourlovelypet.views.fragments;

import android.os.Bundle;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModelProvider;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import java.util.ArrayList;
import java.util.List;
import edu.uoc.tfg.yourlovelypet.databinding.FragmentPetProductsBinding;
import edu.uoc.tfg.yourlovelypet.model.Pet;
import edu.uoc.tfg.yourlovelypet.model.Product;
import edu.uoc.tfg.yourlovelypet.viewModel.MainViewModel;
import edu.uoc.tfg.yourlovelypet.views.adapters.ProductAdapter;
import edu.uoc.tfg.yourlovelypet.views.dialogs.AddPetProductDialogFragment;

public class PetProductsFragment extends Fragment {

    private static final String TAG = "[PetProducts]";
    private static final String ARG_PET_ID = "petId";
    private FragmentPetProductsBinding binding;
    private MainViewModel mainViewModel;
    private RecyclerView recyclerView;
    private ProductAdapter productAdapter;
    private String receivedPetId;
    private List<Product> petProducts = new ArrayList<>();
    private Pet currentPet;

    public PetProductsFragment() {}

    public static PetProductsFragment newInstance(String param) {
        PetProductsFragment fragment = new PetProductsFragment();
        Bundle args = new Bundle();
        args.putString(ARG_PET_ID, param);
        fragment.setArguments(args);
        return fragment;
    }

    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (getArguments() != null) {
            receivedPetId = getArguments().getString(ARG_PET_ID);
            Log.d(TAG,"Receive petId: " + receivedPetId);
        }
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        binding = FragmentPetProductsBinding.inflate(getLayoutInflater(), container, false);
        return binding.getRoot();
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        mainViewModel = new ViewModelProvider(requireActivity()).get(MainViewModel.class);
        currentPet = mainViewModel.getPetById(receivedPetId);
        binding.textViewPetName.setText(currentPet.getPetName());

        // RecyclerView and Adapter (data filtered by petId)
        recyclerView = binding.recyclerViewListOfProducts;
        productAdapter = new ProductAdapter(mainViewModel, getContext());
        recyclerView.setAdapter(productAdapter);
        recyclerView.setLayoutManager(new LinearLayoutManager(getContext()));

        // Observer of products from current user LiveData
        mainViewModel.getProductsFromCurrentUser().observe(getViewLifecycleOwner(), new Observer<List<Product>>() {
            @Override
            public void onChanged(List<Product> products) {

                petProducts.clear(); // To avoid duplicates

                if (products != null) {
                    // Filter products by petId
                    for (Product product : products) {
                        if (product.getPetId().equals(receivedPetId)) {
                            petProducts.add(product);
                        }
                    }
                }

                productAdapter.setData(petProducts);

                if (petProducts.isEmpty()) {
                    binding.textViewNoProducts.setVisibility(View.VISIBLE);
                } else {
                    binding.textViewNoProducts.setVisibility(View.GONE);
                }
            }
        });

        binding.addProductButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                AddPetProductDialogFragment addPetProductDialogFragment = AddPetProductDialogFragment.newInstance(receivedPetId);
                addPetProductDialogFragment.show(getParentFragmentManager(),"addPetProduct");
            }
        });
    }
}