package edu.uoc.tfg.yourlovelypet.views.fragments;

import android.os.Bundle;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import com.bumptech.glide.Glide;
import edu.uoc.tfg.yourlovelypet.databinding.FragmentPhotoBinding;
import edu.uoc.tfg.yourlovelypet.model.Pet;
import edu.uoc.tfg.yourlovelypet.viewModel.MainViewModel;
import edu.uoc.tfg.yourlovelypet.views.dialogs.ConfirmPhotoDeleteDialogFragment;
import edu.uoc.tfg.yourlovelypet.views.dialogs.ConfirmPhotoProfileDialogFragment;

public class PhotoFragment extends Fragment {

    private static final String TAG = "[PhotoFragment]";
    private static final String ARG1 = "imageUrl";
    private static final String ARG2 = "petId";
    private FragmentPhotoBinding binding;
    private MainViewModel mainViewModel;
    private String imageUrl;
    private String petId;
    private Pet currentPet;

    public PhotoFragment() {}

    public static PhotoFragment newInstance(String param1, String param2) {
        PhotoFragment fragment = new PhotoFragment();
        Bundle args = new Bundle();
        args.putString(ARG1, param1);
        args.putString(ARG2, param2);
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (getArguments() != null) {
            imageUrl = getArguments().getString(ARG1);
            petId = getArguments().getString(ARG2);
            Log.d(TAG,"Receive imageUrl: " + imageUrl);
            Log.d(TAG,"Receive petId: " + petId);
        }
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        binding = FragmentPhotoBinding.inflate(getLayoutInflater(), container, false);
        return binding.getRoot();
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        mainViewModel = new ViewModelProvider(requireActivity()).get(MainViewModel.class);
        currentPet = mainViewModel.getPetById(petId);
        Glide.with(requireContext()).load(imageUrl).into(binding.imageView);
        binding.textViewPetName.setText(currentPet.getPetName());

        binding.delete.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                ConfirmPhotoDeleteDialogFragment confirmPhotoDeleteDialogFragment = ConfirmPhotoDeleteDialogFragment.newInstance(imageUrl, petId);
                confirmPhotoDeleteDialogFragment.show(getParentFragmentManager(),"deletePhoto");
            }
        });

        binding.profile.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                ConfirmPhotoProfileDialogFragment confirmPhotoProfileDialogFragment = ConfirmPhotoProfileDialogFragment.newInstance(imageUrl, petId);
                confirmPhotoProfileDialogFragment.show(getParentFragmentManager(),"profilePhoto");
            }
        });

    }
}