package edu.uoc.tfg.yourlovelypet.views.fragments;

import android.os.Bundle;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModelProvider;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import java.util.List;
import edu.uoc.tfg.yourlovelypet.databinding.FragmentProductsBinding;
import edu.uoc.tfg.yourlovelypet.model.Product;
import edu.uoc.tfg.yourlovelypet.model.User;
import edu.uoc.tfg.yourlovelypet.viewModel.MainViewModel;
import edu.uoc.tfg.yourlovelypet.views.adapters.ProductAdapter;
import edu.uoc.tfg.yourlovelypet.views.dialogs.AddProductDialogFragment;

public class ProductsFragment extends Fragment {

    private static final String TAG = "[ProductsFragment]";
    private FragmentProductsBinding binding;
    private MainViewModel mainViewModel;
    private RecyclerView recyclerView;
    private ProductAdapter productAdapter;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        binding = FragmentProductsBinding.inflate(getLayoutInflater(), container, false);
        return binding.getRoot();
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        mainViewModel = new ViewModelProvider(requireActivity()).get(MainViewModel.class);

        // RecyclerView y Adapter
        recyclerView = binding.recyclerViewListOfProducts;
        productAdapter = new ProductAdapter(mainViewModel, getContext());
        recyclerView.setAdapter(productAdapter);
        recyclerView.setLayoutManager(new LinearLayoutManager(getContext()));

        // Observer of current user LiveData
        mainViewModel.getCurrentUser().observe(getViewLifecycleOwner(), new Observer<User>() {
            @Override
            public void onChanged(User user) {
                binding.textViewUserName.setText(user.getUserName());
            }
        });

        // Observer of products from current user LiveData
        mainViewModel.getProductsFromCurrentUser().observe(getViewLifecycleOwner(), new Observer<List<Product>>() {
            @Override
            public void onChanged(List<Product> items) {
                if (items == null || items.isEmpty()) {
                    binding.textViewNoProducts.setVisibility(View.VISIBLE);
                } else {
                    binding.textViewNoProducts.setVisibility(View.GONE);
                }
                productAdapter.setData(items);
            }
        });

        binding.addProductButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                AddProductDialogFragment addProductDialogFragment = new AddProductDialogFragment();
                addProductDialogFragment.show(getParentFragmentManager(),"addProduct");
            }
        });
    }
}