package edu.uoc.tfg.yourlovelypet.views.fragments;

import android.content.res.Resources;
import android.os.Bundle;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModelProvider;
import android.text.TextUtils;
import android.util.Log;
import android.util.Patterns;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.RadioGroup;
import java.util.Locale;
import edu.uoc.tfg.yourlovelypet.R;
import edu.uoc.tfg.yourlovelypet.databinding.FragmentUserBinding;
import edu.uoc.tfg.yourlovelypet.model.User;
import edu.uoc.tfg.yourlovelypet.viewModel.MainViewModel;
import edu.uoc.tfg.yourlovelypet.views.dialogs.LogoutConfirmDialogFragment;

public class UserFragment extends Fragment {

    private static final String TAG = "[UserFragment]";
    private static final String ES = "es";
    private static final String EN = "en";
    private FragmentUserBinding binding;
    private MainViewModel mainViewModel;
    private User currentUser;

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        binding = FragmentUserBinding.inflate(getLayoutInflater(), container, false);
        mainViewModel = new ViewModelProvider(requireActivity()).get(MainViewModel.class);
        return binding.getRoot();
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        currentUser = mainViewModel.getCurrentUser().getValue();
        // TODO maybe check user language and in case it was different to current locale
        //  set locale and recreate MainActivity

        // Set radioButton with current locale language
        Locale currentLocale = getResources().getConfiguration().locale;
        if (currentLocale.getLanguage().equals(ES)) {
            binding.language.check(R.id.spanish);
        } else {
            binding.language.check(R.id.english);
        }

        // Observe currentUser like LiveData
        mainViewModel.getCurrentUser().observe(getViewLifecycleOwner(), new Observer<User>() {
            @Override
            public void onChanged(User user) {
                binding.editUsername.setText(user.getUserName());
                binding.editEmail.setText(user.getUserEmail());
            }
        });

        binding.buttonUpdateName.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Log.d(TAG, "Request to update name from user: " + currentUser.getId());
                String name = String.valueOf(binding.editUsername.getText());
                if (validateName(name)) mainViewModel.updateUserName(currentUser, name);
            }
        });

        binding.buttonUpdateEmail.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Log.d(TAG, "Request to update email from user: " + currentUser.getId());
                String email = String.valueOf(binding.editEmail.getText());
                String password = String.valueOf(binding.editPassword.getText());
                if (validateEmail(email) && validatePassword(password)) mainViewModel.updateUserEmail(currentUser, email, password);
            }
        });

        binding.buttonUpdatePassword.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Log.d(TAG, "Request to update password from user: " + currentUser.getId());
                String password = String.valueOf(binding.editPassword.getText());
                String newPassword = String.valueOf(binding.editNewPassword.getText());
                String repeatNewPassword = String.valueOf(binding.editRepeatNewPassword.getText());
                if (validatePassword(password) && validateNewPassword(newPassword, repeatNewPassword)) mainViewModel.updateUserPassword(password, newPassword);
            }
        });

        binding.logout.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                LogoutConfirmDialogFragment logoutConfirmDialogFragment = new LogoutConfirmDialogFragment();
                logoutConfirmDialogFragment.show(getParentFragmentManager(),"logoutConfirm");
            }
        });

        binding.language.setOnCheckedChangeListener(new RadioGroup.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(RadioGroup group, int checkedId) {
                Resources resources = getResources();
                if ( (checkedId == R.id.english)  ) {
                    mainViewModel.updateUserLanguage(EN, resources);
                    Log.d(TAG, "Request to update user language to english");
                } else if ( (checkedId == R.id.spanish)  ) {
                    mainViewModel.updateUserLanguage(ES, resources);
                    Log.d(TAG, "Request to update user language to spanish");
                }
            }
        });
    }

    private boolean validateName(String name) {
        if (TextUtils.isEmpty(name)) {
            binding.editUsername.setError(getString(R.string.user_error_name_required));
            return false;
        }
        return true;
    }

    private boolean validateEmail(String email) {
        if (TextUtils.isEmpty(email)) {
            binding.editEmail.setError(getString(R.string.user_error_email_required));
            return false;
        }
        if (!Patterns.EMAIL_ADDRESS.matcher(email).matches()) {
            binding.editEmail.setError(getString(R.string.user_error_email_invalid));
            return false;
        }
        return true;
    }

    private boolean validatePassword(String password) {
        if (TextUtils.isEmpty(password)) {
            binding.editPassword.setError(getString(R.string.user_error_password_required));
            return false;
        }
        if (password.length() < 6) {
            binding.editPassword.setError(getString(R.string.user_error_password_length));
            return false;
        }
        return true;
    }

    private boolean validateNewPassword(String newPassword, String confirmNewPassword) {
        if (TextUtils.isEmpty(newPassword)) {
            binding.editNewPassword.setError(getString(R.string.user_error_new_password_required));
            return false;
        }
        if (newPassword.length() < 6) {
            binding.editNewPassword.setError(getString(R.string.user_error_new_password_length));
            return false;
        }
        if (!newPassword.equals(confirmNewPassword)) {
            binding.editRepeatNewPassword.setError(getString(R.string.user_error_new_password_mismatch));
            return false;
        }
        return true;
    }
}